@@ -25,7 +25,7 @@ import locale
 import traceback
 import sys
 import queue
-from typing import NamedTuple, Optional
+from typing import TYPE_CHECKING, NamedTuple, Optional
 
 from .version import ELECTRUM_VERSION
 from . import constants
@@ -33,6 +33,9 @@ from .i18n import _
 from .util import make_aiohttp_session, error_text_str_to_safe_str
 from .logging import describe_os_version, Logger, get_git_version
 
+if TYPE_CHECKING:
+    from .network import ProxySettings
+
 
 class CrashReportResponse(NamedTuple):
     status: Optional[str]
@@ -42,7 +45,6 @@ class CrashReportResponse(NamedTuple):
 
 class BaseCrashReporter(Logger):
     report_server = "https://crashhub.electrum.org"
-    config_key = "show_crash_reporter"
     issue_template = """<h2>Traceback</h2>
 <pre>
 {traceback}
@@ -70,9 +72,14 @@ class BaseCrashReporter(Logger):
         Logger.__init__(self)
         self.exc_args = (exctype, value, tb)
 
-    def send_report(self, asyncio_loop, proxy, *, timeout=None) -> CrashReportResponse:
+    def send_report(self, asyncio_loop, proxy: 'ProxySettings', *, timeout=None) -> CrashReportResponse:
         # FIXME the caller needs to catch generic "Exception", as this method does not have a well-defined API...
-        if constants.net.GENESIS[-4:] not in ["4943", "e26f"] and ".electrum.org" in BaseCrashReporter.report_server:
+        if (constants.net.GENESIS[-4:] not in [
+            "e26f",  # mainnet
+            "4943",  # testnet 3
+            "f043",  # testnet 4
+            "1ef6",  # signet
+        ] and ".electrum.org" in BaseCrashReporter.report_server):
             # Gah! Some kind of altcoin wants to send us crash reports.
             raise Exception(_("Missing report URL."))
         report = self.get_traceback_info()
@@ -99,7 +106,7 @@ class BaseCrashReporter(Logger):
         )
         return ret
 
-    async def do_post(self, proxy, url, data) -> str:
+    async def do_post(self, proxy: 'ProxySettings', url, data) -> str:
         async with make_aiohttp_session(proxy) as session:
             async with session.post(url, data=data, raise_for_status=True) as resp:
                 return await resp.text()
@@ -125,7 +132,7 @@ class BaseCrashReporter(Logger):
             "python_version": sys.version,
             "os": describe_os_version(),
             "wallet_type": "unknown",
-            "locale": locale.getdefaultlocale()[0] or "?",
+            "locale": locale.getlocale()[0] or "?",
             "description": self.get_user_description()
         }
         try:
