@@ -5,13 +5,15 @@ import os
 import sys
 import html
 import threading
-from typing import TYPE_CHECKING, Set
+from functools import partial
+from typing import TYPE_CHECKING, Set, List, Optional, Callable
 
 from PyQt6.QtCore import (pyqtSlot, pyqtSignal, pyqtProperty, QObject, QT_VERSION_STR, PYQT_VERSION_STR,
                           qInstallMessageHandler, QTimer, QSortFilterProxyModel)
-from PyQt6.QtGui import QGuiApplication, QFontDatabase, QScreen
-from PyQt6.QtQml import qmlRegisterType, qmlRegisterUncreatableType, QQmlApplicationEngine
+from PyQt6.QtGui import QGuiApplication
+from PyQt6.QtQml import qmlRegisterType, QQmlApplicationEngine
 
+import electrum
 from electrum import version, constants
 from electrum.i18n import _
 from electrum.logging import Logger, get_logger
@@ -19,6 +21,7 @@ from electrum.bip21 import BITCOIN_BIP21_URI_SCHEME, LIGHTNING_URI_SCHEME
 from electrum.base_crash_reporter import BaseCrashReporter, EarlyExceptionsQueue
 from electrum.network import Network
 from electrum.plugin import run_hook
+from electrum.gui.common_qt.util import get_font_id
 
 from .qeconfig import QEConfig
 from .qedaemon import QEDaemon
@@ -28,7 +31,7 @@ from .qeqr import QEQRParser, QEQRImageProvider, QEQRImageProviderHelper
 from .qeqrscanner import QEQRScanner
 from .qebitcoin import QEBitcoin
 from .qefx import QEFX
-from .qetxfinalizer import QETxFinalizer, QETxRbfFeeBumper, QETxCpfpFeeBumper, QETxCanceller
+from .qetxfinalizer import QETxFinalizer, QETxRbfFeeBumper, QETxCpfpFeeBumper, QETxCanceller, QETxSweepFinalizer, FeeSlider
 from .qeinvoice import QEInvoice, QEInvoiceParser
 from .qerequestdetails import QERequestDetails
 from .qetypes import QEAmount
@@ -38,7 +41,7 @@ from .qechannelopener import QEChannelOpener
 from .qelnpaymentdetails import QELnPaymentDetails
 from .qechanneldetails import QEChannelDetails
 from .qeswaphelper import QESwapHelper
-from .qewizard import QENewWalletWizard, QEServerConnectWizard
+from .qewizard import QENewWalletWizard, QEServerConnectWizard, QETermsOfUseWizard
 from .qemodelfilter import QEFilterProxyModel
 from .qebip39recovery import QEBip39RecoveryListModel
 
@@ -50,7 +53,7 @@ if TYPE_CHECKING:
 
 if 'ANDROID_DATA' in os.environ:
     from jnius import autoclass, cast
-    from android import activity
+    from android import activity, permissions
 
     jpythonActivity = autoclass('org.kivy.android.PythonActivity').mActivity
     jHfc = autoclass('android.view.HapticFeedbackConstants')
@@ -71,21 +74,25 @@ class QEAppController(BaseCrashReporter, QObject):
     sendingBugreportFailure = pyqtSignal(str)
     secureWindowChanged = pyqtSignal()
     wantCloseChanged = pyqtSignal()
+    pluginLoaded = pyqtSignal(str)
+    startupFinished = pyqtSignal()
 
-    def __init__(self, qeapp: 'ElectrumQmlApplication', qedaemon: 'QEDaemon', plugins: 'Plugins'):
+    def __init__(self, qeapp: 'ElectrumQmlApplication', plugins: 'Plugins'):
         BaseCrashReporter.__init__(self, None, None, None)
         QObject.__init__(self)
 
         self._app = qeapp
-        self._qedaemon = qedaemon
         self._plugins = plugins
-        self.config = qedaemon.daemon.config
+        self.config = QEConfig.instance.config
 
         self._crash_user_text = ''
         self._app_started = False
         self._intent = ''
         self._secureWindow = False
 
+        # map of permissions and grant status _after_ asking user
+        self._permissions = {}  # type: dict[str, bool]
+
         # set up notification queue and notification_timer
         self.user_notification_queue = queue.Queue()
         self.user_notification_last_time = 0
@@ -95,7 +102,7 @@ class QEAppController(BaseCrashReporter, QObject):
         self.notification_timer.setInterval(500)  # msec
         self.notification_timer.timeout.connect(self.on_notification_timer)
 
-        self._qedaemon.walletLoaded.connect(self.on_wallet_loaded)
+        QEDaemon.instance.walletLoaded.connect(self.on_wallet_loaded)
 
         self.userNotify.connect(self.doNotify)
 
@@ -105,12 +112,12 @@ class QEAppController(BaseCrashReporter, QObject):
         self._want_close = False
 
     def on_wallet_loaded(self):
-        qewallet = self._qedaemon.currentWallet
+        qewallet = QEDaemon.instance.currentWallet
         if not qewallet:
             return
 
         # register wallet in Exception_Hook
-        Exception_Hook.maybe_setup(config=qewallet.wallet.config, wallet=qewallet.wallet)
+        Exception_Hook.maybe_setup(wallet=qewallet.wallet)
 
         # attach to the wallet user notification events
         # connect only once
@@ -122,10 +129,11 @@ class QEAppController(BaseCrashReporter, QObject):
 
     def on_wallet_usernotify(self, wallet, message):
         self.logger.debug(message)
-        self.user_notification_queue.put((wallet,message))
+        self.user_notification_queue.put((wallet, message))
         if not self.notification_timer.isActive():
             self.logger.debug('starting app notification timer')
             self.notification_timer.start()
+            self.on_notification_timer()
 
     def on_notification_timer(self):
         if self.user_notification_queue.qsize() == 0:
@@ -138,6 +146,13 @@ class QEAppController(BaseCrashReporter, QObject):
             return
         self.user_notification_last_time = now
         self.logger.info("Notifying GUI about new user notifications")
+        # request permission and defer notify until after permission request callback
+        # note: permission request is only shown to user once, so it is safe to request
+        # multiple times
+        if self.isAndroid() and not self.hasPermission(permissions.Permission.POST_NOTIFICATIONS) \
+                and self._permissions.get(permissions.Permission.POST_NOTIFICATIONS) is None:
+            self.request_permission(permissions.Permission.POST_NOTIFICATIONS)
+            return
         try:
             wallet, message = self.user_notification_queue.get_nowait()
             self.userNotify.emit(str(wallet), message)
@@ -145,16 +160,18 @@ class QEAppController(BaseCrashReporter, QObject):
             pass
 
     def doNotify(self, wallet_name, message):
+        self.logger.debug(f'sending push notification to OS: {message=!r}')
         try:
             # TODO: lazy load not in UI thread please
             global notification
             if not notification:
                 from plyer import notification
-            icon = (os.path.dirname(os.path.realpath(__file__))
-                    + '/../icons/electrum.png')
+            icon = os.path.join(
+                os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "icons", "electrum.png",
+            )
             notification.notify('Electrum', message, app_icon=icon, app_name='Electrum')
         except ImportError:
-            self.logger.warning('Notification: needs plyer; `sudo python3 -m pip install plyer`')
+            self.logger.warning('Notification: needs plyer; `python3 -m pip install plyer`')
         except Exception as e:
             self.logger.error(repr(e))
 
@@ -167,6 +184,42 @@ class QEAppController(BaseCrashReporter, QObject):
         except Exception as e:
             self.logger.error(f'unable to bind intent: {repr(e)}')
 
+    @pyqtSlot(str, result=bool)
+    def hasPermission(self, permissionFqcn: str) -> bool:
+        if not self.isAndroid():
+            return True
+        result = permissions.check_permission(permissionFqcn)
+        return result
+
+    def request_permission(self, permissionFqcn: str, permission_result_cb: Optional[Callable] = None):
+        if not self.isAndroid():
+            return True
+        self.logger.debug(f'requesting {permissionFqcn=}')
+        permissions.request_permission(
+            permissionFqcn,
+            callback=partial(self.on_request_permissions_result, permissionFqcn, permission_result_cb)
+        )
+
+    def on_request_permissions_result(
+            self,
+            permission: str,
+            permission_result_cb: Optional[Callable[[bool], None]],
+            permissions: List[str],
+            grant_results: List[bool]
+    ):
+        self.logger.debug(f'on_request_permissions_result, len={len(permissions)}, p={repr(permissions)}, g={repr(grant_results)}')
+        grant_result = None
+        try:
+            grant_result = grant_results[permissions.index(permission)]
+        except ValueError:
+            pass
+
+        if grant_result is not None:
+            self._permissions[permission] = grant_result
+
+        if permission_result_cb:
+            permission_result_cb(grant_result)
+
     def on_new_intent(self, intent):
         if not self._app_started:
             self._intent = intent
@@ -178,8 +231,11 @@ class QEAppController(BaseCrashReporter, QObject):
         if scheme == BITCOIN_BIP21_URI_SCHEME or scheme == LIGHTNING_URI_SCHEME:
             self.uriReceived.emit(data)
 
-    def startupFinished(self):
+    def startup_finished(self):
         self._app_started = True
+        self.startupFinished.emit()
+        for plugin_name in self._plugins.plugins.keys():
+            self.pluginLoaded.emit(plugin_name)
         if self._intent:
             self.on_new_intent(self._intent)
 
@@ -205,37 +261,64 @@ class QEAppController(BaseCrashReporter, QObject):
         it = jIntent.createChooser(sendIntent, cast('java.lang.CharSequence', jString(title)))
         jpythonActivity.startActivity(it)
 
+    @pyqtSlot(result=bool)
+    def isMaxBrightnessOnQrDisplayEnabled(self):
+        return self.config.GUI_QML_SET_MAX_BRIGHTNESS_ON_QR_DISPLAY
+
+    @pyqtSlot()
+    def setMaxScreenBrightness(self):
+        self._set_screen_brightness(1.0)
+
+    @pyqtSlot()
+    def resetScreenBrightness(self):
+        self._set_screen_brightness(-1.0)
+
+    def _set_screen_brightness(self, br: float) -> None:
+        """br is the desired screen brightness, a value in the [0, 1] interval.
+        A negative value, e.g. -1.0, means a "reset" back to the system preferred value.
+        """
+        if not self.isAndroid():
+            return
+        from android.runnable import run_on_ui_thread
+
+        @run_on_ui_thread
+        def set_br():
+            window = jpythonActivity.getWindow()
+            attrs = window.getAttributes()
+            attrs.screenBrightness = br
+            window.setAttributes(attrs)
+        set_br()
+
     @pyqtSlot('QString')
     def textToClipboard(self, text):
         QGuiApplication.clipboard().setText(text)
 
     @pyqtSlot(result='QString')
     def clipboardToText(self):
-        return QGuiApplication.clipboard().text()
+        clip = QGuiApplication.clipboard()
+        return clip.text() if clip.mimeData().hasText() else ''
 
     @pyqtSlot(str, result=QObject)
     def plugin(self, plugin_name):
         self.logger.debug(f'now {self._plugins.count()} plugins loaded')
         plugin = self._plugins.get(plugin_name)
         self.logger.debug(f'plugin with name {plugin_name} is {str(type(plugin))}')
-        if plugin and hasattr(plugin,'so'):
+        if plugin and hasattr(plugin, 'so'):
             return plugin.so
         else:
             self.logger.debug('None!')
             return None
 
-    @pyqtProperty('QVariant', notify=_dummy)
+    @pyqtProperty('QVariantList', notify=_dummy)
     def plugins(self):
         s = []
         for item in self._plugins.descriptions:
-            self.logger.info(item)
             s.append({
                 'name': item,
                 'fullname': self._plugins.descriptions[item]['fullname'],
                 'enabled': bool(self._plugins.get(item))
                 })
 
-        self.logger.debug(f'{str(s)}')
         return s
 
     @pyqtSlot(str, bool)
@@ -263,17 +346,19 @@ class QEAppController(BaseCrashReporter, QObject):
             'reportstring': self.get_report_string()
         }
 
-    @pyqtSlot(object,object,object,object)
-    def crash(self, config, e, text, tb):
-        self.exc_args = (e, text, tb) # for BaseCrashReporter
+    @pyqtSlot(object, object, object)
+    def crash(self, e, text, tb):
+        self.exc_args = (e, text, tb)  # for BaseCrashReporter
         self.showException.emit(self.crashData())
 
-    @pyqtSlot()
-    def sendReport(self):
+    @pyqtSlot(str)
+    def sendReport(self, user_text: str):
+        self._crash_user_text = user_text
         network = Network.get_instance()
         proxy = network.proxy
 
         def report_task():
+            self.logger.debug('starting report_task')
             try:
                 response = BaseCrashReporter.send_report(self, network.asyncio_loop, proxy)
             except Exception as e:
@@ -295,16 +380,12 @@ class QEAppController(BaseCrashReporter, QObject):
     def showNever(self):
         self.config.SHOW_CRASH_REPORTER = False
 
-    @pyqtSlot(str)
-    def setCrashUserText(self, text):
-        self._crash_user_text = text
-
     def _get_traceback_str_to_display(self) -> str:
         # The msg_box that shows the report uses rich_text=True, so
         # if traceback contains special HTML characters, e.g. '<',
         # they need to be escaped to avoid formatting issues.
         traceback_str = super()._get_traceback_str_to_display()
-        return html.escape(traceback_str).replace('&#x27;','&apos;')
+        return html.escape(traceback_str).replace('&#x27;', '&apos;')
 
     def get_user_description(self):
         return self._crash_user_text
@@ -344,11 +425,10 @@ class ElectrumQmlApplication(QGuiApplication):
 
         self.logger = get_logger(__name__)
 
-        ElectrumQmlApplication._daemon = daemon
-
         # TODO QT6 order of declaration is important now?
         qmlRegisterType(QEAmount, 'org.electrum', 1, 0, 'Amount')
         qmlRegisterType(QENewWalletWizard, 'org.electrum', 1, 0, 'QNewWalletWizard')
+        qmlRegisterType(QETermsOfUseWizard, 'org.electrum', 1, 0, 'QTermsOfUseWizard')
         qmlRegisterType(QEServerConnectWizard, 'org.electrum', 1, 0, 'QServerConnectWizard')
         qmlRegisterType(QEFilterProxyModel, 'org.electrum', 1, 0, 'FilterProxyModel')
         qmlRegisterType(QSortFilterProxyModel, 'org.electrum', 1, 0, 'QSortFilterProxyModel')
@@ -371,8 +451,9 @@ class ElectrumQmlApplication(QGuiApplication):
         qmlRegisterType(QETxRbfFeeBumper, 'org.electrum', 1, 0, 'TxRbfFeeBumper')
         qmlRegisterType(QETxCpfpFeeBumper, 'org.electrum', 1, 0, 'TxCpfpFeeBumper')
         qmlRegisterType(QETxCanceller, 'org.electrum', 1, 0, 'TxCanceller')
+        qmlRegisterType(QETxSweepFinalizer, 'org.electrum', 1, 0, 'SweepFinalizer')
         qmlRegisterType(QEBip39RecoveryListModel, 'org.electrum', 1, 0, 'Bip39RecoveryListModel')
-
+        qmlRegisterType(FeeSlider, 'org.electrum', 1, 0, 'FeeSlider')
         # TODO QT6: these were declared as uncreatable, but that doesn't seem to work for pyqt6
         # qmlRegisterUncreatableType(QEAmount, 'org.electrum', 1, 0, 'Amount', 'Amount can only be used as property')
         # qmlRegisterUncreatableType(QENewWalletWizard, 'org.electrum', 1, 0, 'QNewWalletWizard', 'QNewWalletWizard can only be used as property')
@@ -391,25 +472,25 @@ class ElectrumQmlApplication(QGuiApplication):
 
         # add a monospace font as we can't rely on device having one
         self.fixedFont = 'PT Mono'
-        not_loaded = QFontDatabase.addApplicationFont('electrum/gui/qml/fonts/PTMono-Regular.ttf') < 0
-        not_loaded = QFontDatabase.addApplicationFont('electrum/gui/qml/fonts/PTMono-Bold.ttf') < 0 and not_loaded
+        not_loaded = get_font_id('PTMono-Regular.ttf') < 0
+        not_loaded = get_font_id('PTMono-Bold.ttf') < 0 and not_loaded
         if not_loaded:
             self.logger.warning('Could not load font PT Mono')
             self.fixedFont = 'Monospace' # hope for the best
 
         self.context = self.engine.rootContext()
         self.plugins = plugins
-        self._qeconfig = QEConfig(config)
-        self._qenetwork = QENetwork(daemon.network, self._qeconfig)
+        self.config = QEConfig(config)
+        self.network = QENetwork(daemon.network)
         self.daemon = QEDaemon(daemon, self.plugins)
-        self.appController = QEAppController(self, self.daemon, self.plugins)
-        self._maxAmount = QEAmount(is_max=True)
+        self.appController = QEAppController(self, self.plugins)
+        self.maxAmount = QEAmount(is_max=True)
         self.context.setContextProperty('AppController', self.appController)
-        self.context.setContextProperty('Config', self._qeconfig)
-        self.context.setContextProperty('Network', self._qenetwork)
+        self.context.setContextProperty('Config', self.config)
+        self.context.setContextProperty('Network', self.network)
         self.context.setContextProperty('Daemon', self.daemon)
         self.context.setContextProperty('FixedFont', self.fixedFont)
-        self.context.setContextProperty('MAX', self._maxAmount)
+        self.context.setContextProperty('MAX', self.maxAmount)
         self.context.setContextProperty('QRIP', self.qr_ip_h)
         self.context.setContextProperty('BUILD', {
             'electrum_version': version.ELECTRUM_VERSION,
@@ -417,8 +498,14 @@ class ElectrumQmlApplication(QGuiApplication):
             'qt_version': QT_VERSION_STR,
             'pyqt_version': PYQT_VERSION_STR
         })
+        self.context.setContextProperty('UI_UNIT_NAME', {
+            "FEERATE_SAT_PER_VBYTE": electrum.util.UI_UNIT_NAME_FEERATE_SAT_PER_VBYTE,
+            "FEERATE_SAT_PER_VB":    electrum.util.UI_UNIT_NAME_FEERATE_SAT_PER_VB,
+            "TXSIZE_VBYTES":         electrum.util.UI_UNIT_NAME_TXSIZE_VBYTES,
+            "MEMPOOL_MB":            electrum.util.UI_UNIT_NAME_MEMPOOL_MB,
+        })
 
-        self.plugins.load_plugin('trustedcoin')
+        self.plugins.load_plugin_by_name('trustedcoin')
 
         qInstallMessageHandler(self.message_handler)
 
@@ -428,10 +515,11 @@ class ElectrumQmlApplication(QGuiApplication):
     # slot is called after loading root QML. If object is None, it has failed.
     @pyqtSlot('QObject*', 'QUrl')
     def objectCreated(self, object, url):
+        self.engine.objectCreated.disconnect(self.objectCreated)
         if object is None:
             self._valid = False
-        self.engine.objectCreated.disconnect(self.objectCreated)
-        self.appController.startupFinished()
+        else:
+            self.appController.startup_finished()
 
     def message_handler(self, line, funct, file):
         # filter out common harmless messages
@@ -441,33 +529,33 @@ class ElectrumQmlApplication(QGuiApplication):
 
 
 class Exception_Hook(QObject, Logger):
-    _report_exception = pyqtSignal(object, object, object, object)
+    _report_exception = pyqtSignal(object, object, object)
 
     _INSTANCE = None  # type: Optional[Exception_Hook]  # singleton
 
-    def __init__(self, *, config: 'SimpleConfig', slot):
+    def __init__(self, *, slot):
         QObject.__init__(self)
         Logger.__init__(self)
         assert self._INSTANCE is None, "Exception_Hook is supposed to be a singleton"
-        self.config = config
         self.wallet_types_seen = set()  # type: Set[str]
 
         sys.excepthook = self.handler
         threading.excepthook = self.handler
 
-        self._report_exception.connect(slot)
+        if slot:
+            self._report_exception.connect(slot)
         EarlyExceptionsQueue.set_hook_as_ready()
 
     @classmethod
-    def maybe_setup(cls, *, config: 'SimpleConfig', wallet: 'Abstract_Wallet' = None, slot = None) -> None:
-        if not config.SHOW_CRASH_REPORTER:
+    def maybe_setup(cls, *, wallet: 'Abstract_Wallet' = None, slot=None) -> None:
+        if not QEConfig.instance.config.SHOW_CRASH_REPORTER:
             EarlyExceptionsQueue.set_hook_as_ready()  # flush already queued exceptions
             return
         if not cls._INSTANCE:
-            cls._INSTANCE = Exception_Hook(config=config, slot=slot)
+            cls._INSTANCE = Exception_Hook(slot=slot)
         if wallet:
             cls._INSTANCE.wallet_types_seen.add(wallet.wallet_type)
 
     def handler(self, *exc_info):
         self.logger.error('exception caught by crash reporter', exc_info=exc_info)
-        self._report_exception.emit(self.config, *exc_info)
+        self._report_exception.emit(*exc_info)
