@@ -1,3 +1,5 @@
+import copy
+import threading
 from enum import IntEnum
 from typing import Optional, Dict, Any
 from urllib.parse import urlparse
@@ -6,16 +8,21 @@ from PyQt6.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject, pyqtEnum,
 
 from electrum.i18n import _
 from electrum.logging import get_logger
-from electrum.invoices import (Invoice, PR_UNPAID, PR_EXPIRED, PR_UNKNOWN, PR_PAID, PR_INFLIGHT,
-                               PR_FAILED, PR_ROUTING, PR_UNCONFIRMED, PR_BROADCASTING, PR_BROADCAST, LN_EXPIRY_NEVER)
+from electrum.invoices import (
+    Invoice, PR_UNPAID, PR_EXPIRED, PR_UNKNOWN, PR_PAID, PR_INFLIGHT, PR_FAILED, PR_ROUTING, PR_UNCONFIRMED,
+    PR_BROADCASTING, PR_BROADCAST, LN_EXPIRY_NEVER
+)
 from electrum.transaction import PartialTxOutput, TxOutput
 from electrum.lnutil import format_short_channel_id
-from electrum.bitcoin import COIN
+from electrum.bitcoin import COIN, address_to_script
 from electrum.paymentrequest import PaymentRequest
-from electrum.payment_identifier import (PaymentIdentifier, PaymentIdentifierState, PaymentIdentifierType)
+from electrum.payment_identifier import PaymentIdentifier, PaymentIdentifierState, PaymentIdentifierType
+from electrum.network import Network
+
 from .qetypes import QEAmount
 from .qewallet import QEWallet
 from .util import status_update_timer_interval, QtEventListener, event_listener
+from ...fee_policy import FeePolicy
 
 
 class QEInvoice(QObject, QtEventListener):
@@ -42,6 +49,7 @@ class QEInvoice(QObject, QtEventListener):
     invoiceChanged = pyqtSignal()
     invoiceSaved = pyqtSignal([str], arguments=['key'])
     amountOverrideChanged = pyqtSignal()
+    maxAmountMessage = pyqtSignal([str], arguments=['message'])
 
     def __init__(self, parent=None):
         super().__init__(parent)
@@ -52,7 +60,7 @@ class QEInvoice(QObject, QtEventListener):
         self._canPay = False
         self._key = None
         self._invoiceType = QEInvoice.Type.Invalid
-        self._effectiveInvoice = None
+        self._effectiveInvoice = None  # type: Optional[Invoice]
         self._userinfo = ''
         self._lnprops = {}
         self._amount = QEAmount()
@@ -64,6 +72,8 @@ class QEInvoice(QObject, QtEventListener):
 
         self._amountOverride.valueChanged.connect(self._on_amountoverride_value_changed)
 
+        self._updating_max = False
+
         self.register_callbacks()
         self.destroyed.connect(lambda: self.on_destroy())
 
@@ -94,6 +104,7 @@ class QEInvoice(QObject, QtEventListener):
     @event_listener
     def on_event_channel(self, wallet, channel):
         if self._wallet and wallet == self._wallet.wallet:
+            self.update_userinfo()
             self.determine_can_pay()
 
     walletChanged = pyqtSignal()
@@ -144,7 +155,7 @@ class QEInvoice(QObject, QtEventListener):
         return self._amountOverride
 
     @amountOverride.setter
-    def amountOverride(self, new_amount):
+    def amountOverride(self, new_amount: QEAmount):
         self._logger.debug(f'set new override amount {repr(new_amount)}')
         self._amountOverride.copyFrom(new_amount)
         self.amountOverrideChanged.emit()
@@ -159,6 +170,9 @@ class QEInvoice(QObject, QtEventListener):
     def status(self):
         if not self._effectiveInvoice:
             return PR_UNKNOWN
+        if self.invoiceType == QEInvoice.Type.OnchainInvoice and self._effectiveInvoice.get_amount_sat() == 0:
+            # no amount set, not a final invoice, get_invoice_status would be wrong
+            return PR_UNPAID
         return self._wallet.wallet.get_invoice_status(self._effectiveInvoice)
 
     @pyqtProperty(str, notify=statusChanged)
@@ -202,14 +216,11 @@ class QEInvoice(QObject, QtEventListener):
 
     @key.setter
     def key(self, key):
-        if self._key != key:
-            self._key = key
-            if self._effectiveInvoice and self._effectiveInvoice.get_id() == key:
-                return
-            invoice = self._wallet.wallet.get_invoice(key)
-            self._logger.debug(f'invoice from key {key}: {repr(invoice)}')
-            self.set_effective_invoice(invoice)
-            self.keyChanged.emit()
+        self._key = key
+        invoice = copy.copy(self._wallet.wallet.get_invoice(key))  # copy, so any mutations stay out of wallet invoice list
+        self._logger.debug(f'invoice from key {key}: {repr(invoice)}')
+        self.set_effective_invoice(invoice)
+        self.keyChanged.emit()
 
     userinfoChanged = pyqtSignal()
     @pyqtProperty(str, notify=userinfoChanged)
@@ -301,37 +312,35 @@ class QEInvoice(QObject, QtEventListener):
         if amount.isEmpty and self.status == PR_UNPAID:  # unspecified amount
             return
 
+        def userinfo_for_invoice_status(status: int) -> str:
+            return {
+                PR_EXPIRED: _('This invoice has expired'),
+                PR_PAID: _('This invoice was already paid'),
+                PR_INFLIGHT: _('Payment in progress...'),
+                PR_ROUTING: _('Payment in progress...'),
+                PR_BROADCASTING: _('Payment in progress...') + ' (' + _('broadcasting') + ')',
+                PR_BROADCAST:  _('Payment in progress...') + ' (' + _('broadcast successfully') + ')',
+                PR_UNCONFIRMED: _('Payment in progress...') + ' (' + _('waiting for confirmation') + ')',
+                PR_UNKNOWN: _('Invoice has unknown status'),
+            }[status]
+
         if self.invoiceType == QEInvoice.Type.LightningInvoice:
             if self.status in [PR_UNPAID, PR_FAILED]:
                 if self.get_max_spendable_lightning() >= amount.satsInt:
                     lnaddr = self._effectiveInvoice._lnaddr
                     if lnaddr.amount and amount.satsInt < lnaddr.amount * COIN:
                         self.userinfo = _('Cannot pay less than the amount specified in the invoice')
-                elif self.address and self.get_max_spendable_onchain() < amount.satsInt:
-                    # TODO: validate address?
-                    # TODO: subtract fee?
+                elif not self.address or self.get_max_spendable_onchain() < amount.satsInt:
+                    # TODO: for onchain: validate address? subtract fee?
                     self.userinfo = _('Insufficient balance')
             else:
-                self.userinfo = {
-                        PR_EXPIRED: _('This invoice has expired'),
-                        PR_PAID: _('This invoice was already paid'),
-                        PR_INFLIGHT: _('Payment in progress...'),
-                        PR_ROUTING: _('Payment in progress'),
-                        PR_UNKNOWN: _('Invoice has unknown status'),
-                    }[self.status]
+                self.userinfo = userinfo_for_invoice_status(self.status)
         elif self.invoiceType == QEInvoice.Type.OnchainInvoice:
             if self.status in [PR_UNPAID, PR_FAILED]:
                 if not ((amount.isMax and self.get_max_spendable_onchain() > 0) or (self.get_max_spendable_onchain() >= amount.satsInt)):
                     self.userinfo = _('Insufficient balance')
             else:
-                self.userinfo = {
-                        PR_EXPIRED: _('This invoice has expired'),
-                        PR_PAID: _('This invoice was already paid'),
-                        PR_BROADCASTING: _('Payment in progress...') + ' (' + _('broadcasting') + ')',
-                        PR_BROADCAST:  _('Payment in progress...') + ' (' + _('broadcast successfully') + ')',
-                        PR_UNCONFIRMED: _('Payment in progress...') + ' (' + _('waiting for confirmation') + ')',
-                        PR_UNKNOWN: _('Invoice has unknown status'),
-                    }[self.status]
+                self.userinfo = userinfo_for_invoice_status(self.status)
 
     def determine_can_pay(self):
         self.canPay = False
@@ -377,22 +386,52 @@ class QEInvoice(QObject, QtEventListener):
         if self.invoiceType != QEInvoice.Type.LightningInvoice:
             raise Exception('payLightningInvoice can only pay lightning invoices')
 
+        amount_msat = None
         if self.amount.isEmpty:
             if self.amountOverride.isEmpty:
                 raise Exception('can not pay 0 amount')
-            self._effectiveInvoice.set_amount_msat(self.amountOverride.satsInt * 1000)
+            amount_msat = self.amountOverride.satsInt * 1000
 
-        self._wallet.pay_lightning_invoice(self._effectiveInvoice)
+        self._wallet.pay_lightning_invoice(self._effectiveInvoice, amount_msat)
 
     def get_max_spendable_onchain(self):
-        spendable = self._wallet.confirmedBalance.satsInt
-        if not self._wallet.wallet.config.WALLET_SPEND_CONFIRMED_ONLY:
-            spendable += self._wallet.unconfirmedBalance.satsInt
-        return spendable
+        return self._wallet.wallet.get_spendable_balance_sat()
 
     def get_max_spendable_lightning(self):
         return self._wallet.wallet.lnworker.num_sats_can_send() if self._wallet.wallet.lnworker else 0
 
+    @pyqtSlot()
+    def updateMaxAmount(self):
+        if self._updating_max:
+            return
+
+        assert self.invoiceType == QEInvoice.Type.OnchainInvoice
+
+        # only single address invoice supported
+        invoice_address = self._effectiveInvoice.get_address()
+
+        self._updating_max = True
+
+        def calc_max(address):
+            try:
+                outputs = [PartialTxOutput(scriptpubkey=address_to_script(address), value='!')]
+                make_tx = lambda fee_policy, *, confirmed_only=False: self._wallet.wallet.make_unsigned_transaction(
+                    coins=self._wallet.wallet.get_spendable_coins(None),
+                    outputs=outputs,
+                    fee_policy=fee_policy,
+                    is_sweep=False)
+                amount, message = self._wallet.determine_max(mktx=make_tx)
+                if amount is None:
+                    self._amountOverride.isMax = False
+                else:
+                    self._amountOverride.satsInt = amount
+                if message:
+                    self.maxAmountMessage.emit(message)
+            finally:
+                self._updating_max = False
+
+        threading.Thread(target=calc_max, args=(invoice_address,), daemon=True).start()
+
 
 class QEInvoiceParser(QEInvoice):
     _logger = get_logger(__name__)
@@ -464,6 +503,7 @@ class QEInvoiceParser(QEInvoice):
         self._logger.debug('setValidLightningInvoice')
         if not invoice.is_lightning():
             raise Exception('unexpected Onchain invoice')
+        self._key = invoice.get_id()
         self.set_effective_invoice(invoice)
 
     def setValidLNURLPayRequest(self):
@@ -482,7 +522,7 @@ class QEInvoiceParser(QEInvoice):
 
     def _bip70_payment_request_resolved(self, pr: 'PaymentRequest'):
         self._logger.debug('resolved payment request')
-        if pr.verify():
+        if Network.run_from_another_thread(pr.verify()):
             invoice = Invoice.from_bip70_payreq(pr, height=0)
             if self._wallet.wallet.get_invoice_status(invoice) == PR_PAID:
                 self.validationError.emit('unknown', _('Invoice already paid'))
@@ -574,7 +614,7 @@ class QEInvoiceParser(QEInvoice):
     def resolve_pi(self):
         assert self._pi.need_resolve()
 
-        def on_finished(pi):
+        def on_finished(pi: PaymentIdentifier):
             self._busy = False
             self.busyChanged.emit()
 
@@ -582,7 +622,7 @@ class QEInvoiceParser(QEInvoice):
                 if pi.type in [PaymentIdentifierType.EMAILLIKE, PaymentIdentifierType.DOMAINLIKE]:
                     msg = _('Could not resolve address')
                 elif pi.type == PaymentIdentifierType.LNURLP:
-                    msg = _('Could not resolve LNURL')
+                    msg = _('Could not resolve LNURL') + "\n\n" + pi.get_error()
                 elif pi.type == PaymentIdentifierType.BIP70:
                     msg = _('Could not resolve BIP70 payment request: {}').format(pi.error)
                 else:
@@ -666,6 +706,6 @@ class QEInvoiceParser(QEInvoice):
         self.canSave = False
 
         self._wallet.wallet.save_invoice(self._effectiveInvoice)
-        self.key = self._effectiveInvoice.get_id()
-        self._wallet.invoiceModel.addInvoice(self.key)
-        self.invoiceSaved.emit(self.key)
+        self._key = self._effectiveInvoice.get_id()
+        self._wallet.invoiceModel.addInvoice(self._key)
+        self.invoiceSaved.emit(self._key)
