@@ -4,22 +4,25 @@ from PyQt6.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject
 
 from electrum.i18n import _
 from electrum.logging import get_logger
+from electrum.bitcoin import DummyAddress
 from electrum.util import format_time, TxMinedInfo
-from electrum.transaction import tx_from_any, Transaction, PartialTxInput, Sighash, PartialTransaction, TxOutpoint
+from electrum.transaction import tx_from_any, Transaction, PartialTransaction
 from electrum.network import Network
 from electrum.address_synchronizer import TX_HEIGHT_UNCONF_PARENT, TX_HEIGHT_UNCONFIRMED, TX_HEIGHT_FUTURE
 from electrum.wallet import TxSighashDanger
+from electrum.fee_policy import FeePolicy
 
 from .qewallet import QEWallet
 from .qetypes import QEAmount
-from .util import QtEventListener, event_listener
+from .util import QtEventListener, qt_event_listener
 
 
 class QETxDetails(QObject, QtEventListener):
     _logger = get_logger(__name__)
 
     confirmRemoveLocalTx = pyqtSignal([str], arguments=['message'])
-    saveTxError = pyqtSignal([str,str], arguments=['code', 'message'])
+    txRemoved = pyqtSignal()
+    saveTxError = pyqtSignal([str, str], arguments=['code', 'message'])
     saveTxSuccess = pyqtSignal()
 
     detailsChanged = pyqtSignal()
@@ -56,6 +59,7 @@ class QETxDetails(QObject, QtEventListener):
         self._is_complete = False
         self._is_mined = False
         self._is_rbf_enabled = False
+        self._is_removed = False
         self._lock_delay = 0
         self._sighash_danger = TxSighashDanger()
 
@@ -71,18 +75,26 @@ class QETxDetails(QObject, QtEventListener):
     def on_destroy(self):
         self.unregister_callbacks()
 
-    @event_listener
+    @qt_event_listener
     def on_event_verified(self, wallet, txid, info):
         if wallet == self._wallet.wallet and txid == self._txid:
             self._logger.debug(f'verified event for our txid {txid}')
             self.update()
 
-    @event_listener
+    @qt_event_listener
     def on_event_new_transaction(self, wallet, tx):
         if wallet == self._wallet.wallet and tx.txid() == self._txid:
             self._logger.debug(f'new_transaction event for our txid {self._txid}')
             self.update()
 
+    @qt_event_listener
+    def on_event_removed_transaction(self, wallet, tx):
+        if wallet == self._wallet.wallet and tx.txid() == self._txid:
+            self._logger.debug(f'removed my transaction {tx.txid()}')
+            self._is_removed = True
+            self.update()
+            self.txRemoved.emit()
+
     walletChanged = pyqtSignal()
     @pyqtProperty(QEWallet, notify=walletChanged)
     def wallet(self):
@@ -175,6 +187,10 @@ class QETxDetails(QObject, QtEventListener):
     def isMined(self):
         return self._is_mined
 
+    @pyqtProperty(bool, notify=detailsChanged)
+    def isRemoved(self):
+        return self._is_removed
+
     @pyqtProperty(str, notify=detailsChanged)
     def mempoolDepth(self):
         return self._mempool_depth
@@ -258,6 +274,20 @@ class QETxDetails(QObject, QtEventListener):
     def update(self, from_txid: bool = False):
         assert self._wallet
 
+        if self._is_removed:
+            self._logger.debug('tx removed, disable gui options')
+            self._can_broadcast = False
+            self._can_bump = False
+            self._can_dscancel = False
+            self._can_cpfp = False
+            self._can_save_as_local = False
+            self._can_remove = False
+            self._can_sign = False
+            self._mempool_depth = ''
+            self._status = _('removed')
+            self.detailsChanged.emit()
+            return
+
         if from_txid:
             self._tx = self._wallet.wallet.db.get_transaction(self._txid)
             assert self._tx is not None, f'unknown txid "{self._txid}"'
@@ -270,12 +300,16 @@ class QETxDetails(QObject, QtEventListener):
             Network.run_from_another_thread(
                 self._tx.add_info_from_network(self._wallet.wallet.network, timeout=10))  # FIXME is this needed?...
 
+        sm = self._wallet.wallet.lnworker.swap_manager if self._wallet.wallet.lnworker else None
+
         self._inputs = list(map(lambda x: {
             'short_id': x.prevout.short_name(),
             'value': x.value_sats(),
             'address': x.address,
             'is_mine': self._wallet.wallet.is_mine(x.address),
-            'is_change': self._wallet.wallet.is_change(x.address)
+            'is_change': self._wallet.wallet.is_change(x.address),
+            'is_swap': False if not sm else sm.is_lockup_address_for_a_swap(x.address) or x.address == DummyAddress.SWAP,
+            'is_accounting': self._wallet.wallet.is_accounting_address(x.address)
         }, self._tx.inputs()))
         self._outputs = list(map(lambda x: {
             'address': x.get_ui_address_str(),
@@ -283,7 +317,9 @@ class QETxDetails(QObject, QtEventListener):
             'short_id': '',  # TODO
             'is_mine': self._wallet.wallet.is_mine(x.get_ui_address_str()),
             'is_change': self._wallet.wallet.is_change(x.get_ui_address_str()),
-            'is_billing': self._wallet.wallet.is_billing_address(x.get_ui_address_str())
+            'is_billing': self._wallet.wallet.is_billing_address(x.get_ui_address_str()),
+            'is_swap': False if not sm else sm.is_lockup_address_for_a_swap(x.get_ui_address_str()) or x.get_ui_address_str() == DummyAddress.SWAP,
+            'is_accounting': self._wallet.wallet.is_accounting_address(x.get_ui_address_str())
         }, self._tx.outputs()))
 
         txinfo = self._wallet.wallet.get_tx_info(self._tx)
@@ -315,7 +351,7 @@ class QETxDetails(QObject, QtEventListener):
             self.update_mined_status(txinfo.tx_mined_status)
         else:
             if txinfo.tx_mined_status.height in [TX_HEIGHT_UNCONFIRMED, TX_HEIGHT_UNCONF_PARENT]:
-                self._mempool_depth = self._wallet.wallet.config.depth_tooltip(txinfo.mempool_depth_bytes)
+                self._mempool_depth = FeePolicy.depth_tooltip(txinfo.mempool_depth_bytes)
                 self._in_mempool = True
             elif txinfo.tx_mined_status.height == TX_HEIGHT_FUTURE:
                 self._lock_delay = txinfo.tx_mined_status.wanted_height - self._wallet.wallet.adb.get_local_height()
@@ -323,30 +359,23 @@ class QETxDetails(QObject, QtEventListener):
                 self._sighash_danger = self._wallet.wallet.check_sighash(self._tx)
 
         if self._wallet.wallet.lnworker:
-            # Calling lnworker.get_onchain_history and wallet.get_full_history here
-            # is inefficient. We should probably pass the tx_item to the constructor.
-            lnworker_history = self._wallet.wallet.lnworker.get_onchain_history()
-            if self._txid in lnworker_history:
-                item = lnworker_history[self._txid]
-                group_id = item.get('group_id')
-                if group_id:
-                    full_history = self._wallet.wallet.get_full_history()
-                    group_item = full_history['group:' + group_id]
-                    self._lnamount.satsInt = int(group_item['ln_value'].value)
-                else:
-                    self._lnamount.satsInt = int(item['amount_msat'] / 1000)
-            else:
-                self._lnamount.satsInt = 0
+            # Calling wallet.get_full_history here is inefficient.
+            # We should probably pass the tx_item to the constructor.
+            full_history = self._wallet.wallet.get_full_history()
+            item = full_history.get('group:' + self._txid)
+            self._lnamount.satsInt = int(item['ln_value'].value) if item else 0
+        else:
+            self._lnamount.satsInt = 0
 
         self._is_complete = self._tx.is_complete()
         self._is_rbf_enabled = self._tx.is_rbf_enabled()
         self._is_unrelated = txinfo.amount is None and self._lnamount.isEmpty
         self._is_lightning_funding_tx = txinfo.is_lightning_funding_tx
         self._can_broadcast = txinfo.can_broadcast
-        self._can_bump = txinfo.can_bump and not txinfo.can_remove
-        self._can_dscancel = txinfo.can_dscancel and not txinfo.can_remove
-        self._can_cpfp = txinfo.can_cpfp and not txinfo.can_remove
-        self._can_save_as_local = txinfo.can_save_as_local and not txinfo.can_remove
+        self._can_bump = txinfo.can_bump
+        self._can_dscancel = txinfo.can_dscancel
+        self._can_cpfp = txinfo.can_cpfp
+        self._can_save_as_local = txinfo.can_save_as_local
         self._can_remove = txinfo.can_remove
         self._can_sign = (
             not self._is_complete
@@ -356,9 +385,11 @@ class QETxDetails(QObject, QtEventListener):
 
         self.detailsChanged.emit()
 
-        if self._label != txinfo.label:
-            self._label = txinfo.label
-            self.labelChanged.emit()
+        if self._txid:
+            label = self._wallet.wallet.get_label_for_txid(self._txid)
+            if self._label != label:
+                self._label = label
+                self.labelChanged.emit()
 
     def update_mined_status(self, tx_mined_info: TxMinedInfo):
         self._mempool_depth = ''
@@ -388,8 +419,10 @@ class QETxDetails(QObject, QtEventListener):
         if broadcast:
             self._wallet.broadcastSucceeded.connect(self.onBroadcastSucceeded)
             self._wallet.broadcastFailed.connect(self.onBroadcastFailed)
+            self._wallet.sign_and_broadcast(self._tx, on_success=self.on_signed_tx)
+        else:
+            self._wallet.sign(self._tx, on_success=self.on_signed_tx)
 
-        self._wallet.sign(self._tx, broadcast=broadcast, on_success=self.on_signed_tx)
         # side-effect: signing updates self._tx
         # we rely on this for broadcast
 
@@ -426,7 +459,7 @@ class QETxDetails(QObject, QtEventListener):
         self._can_broadcast = False
         self.detailsChanged.emit()
 
-    @pyqtSlot(str,str,str)
+    @pyqtSlot(str, str, str)
     def onBroadcastFailed(self, txid, code, reason):
         if txid != self._txid:
             return
@@ -461,7 +494,6 @@ class QETxDetails(QObject, QtEventListener):
 
         # NOTE: from here, the tx/txid is unknown and all properties are invalid.
         # UI should close TxDetails and avoid interacting with this qetxdetails instance.
-        self._txid = None
         self._tx = None
 
     @pyqtSlot()
@@ -477,4 +509,5 @@ class QETxDetails(QObject, QtEventListener):
     @pyqtSlot(result='QVariantList')
     def getSerializedTx(self):
         txqr = self._tx.to_qr_data()
-        return [str(self._tx), txqr[0], txqr[1]]
+        label = self._wallet.wallet.get_label_for_txid(self._tx.txid())
+        return [str(self._tx), txqr[0], txqr[1], label]
