@@ -1,20 +1,26 @@
-from PyQt5.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject
+from PyQt6.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject
 
 from electrum.logging import get_logger
 from electrum.i18n import _
 
-# container for satoshi amounts that can be passed around more
-# easily between python, QML-property and QML-javascript contexts
-# QML 'int' is 32 bit signed, so overflows on satoshi amounts
-# QML 'quint64' and 'qint64' can be used, but this breaks
-# down when passing through property bindings
-# should also capture millisats amounts and MAX/'!' indicators
-# and (unformatted) string representations
 
 class QEAmount(QObject):
+    """Container for bitcoin amounts that can be passed around more
+       easily between python, QML-property and QML-javascript contexts.
+       Note: millisat and sat amounts are not synchronized!
+
+       QML type 'int' in property definitions is 32 bit signed, so will overflow easily
+       on (milli)satoshi amounts! 'int' in QML-javascript seems to be larger than 32 bit, and
+       can be used to store q(u)int64 types.
+
+       QML 'quint64' and 'qint64' can be used, but be aware these will in some cases be downcast
+       by QML to 'int' (e.g. when using the property in a property binding, _even_ when a binding
+       is done between two q(u)int64 properties (at least up until Qt6.4))
+    """
+
     _logger = get_logger(__name__)
 
-    def __init__(self, *, amount_sat: int = 0, amount_msat: int = 0, is_max: bool = False, from_invoice = None, parent=None):
+    def __init__(self, *, amount_sat: int = 0, amount_msat: int = 0, is_max: bool = False, from_invoice=None, parent=None):
         super().__init__(parent)
         self._amount_sat = int(amount_sat) if amount_sat is not None else None
         self._amount_msat = int(amount_msat) if amount_msat is not None else None
@@ -31,7 +37,7 @@ class QEAmount(QObject):
 
     @pyqtProperty('qint64', notify=valueChanged)
     def satsInt(self):
-        if self._amount_sat is None: # should normally be defined when accessing this property
+        if self._amount_sat is None:  # should normally be defined when accessing this property
             self._logger.warning('amount_sat is undefined, returning 0')
             return 0
         return self._amount_sat
@@ -44,7 +50,7 @@ class QEAmount(QObject):
 
     @pyqtProperty('qint64', notify=valueChanged)
     def msatsInt(self):
-        if self._amount_msat is None: # should normally be defined when accessing this property
+        if self._amount_msat is None:  # should normally be defined when accessing this property
             self._logger.warning('amount_msat is undefined, returning 0')
             return 0
         return self._amount_msat
@@ -86,7 +92,7 @@ class QEAmount(QObject):
 
     def copyFrom(self, amount):
         if not amount:
-            self._logger.warning('copyFrom with None argument. assuming 0') # TODO
+            self._logger.warning('copyFrom with None argument. assuming 0')  # TODO
             amount = QEAmount()
         self.satsInt = amount.satsInt
         self.msatsInt = amount.msatsInt
