@@ -3,7 +3,7 @@ import base64
 import queue
 import threading
 import time
-from typing import TYPE_CHECKING, Callable
+from typing import TYPE_CHECKING, Callable, Optional, Any, Tuple
 from functools import partial
 
 from PyQt6.QtCore import pyqtProperty, pyqtSignal, pyqtSlot, QObject, QTimer
@@ -13,10 +13,13 @@ from electrum.invoices import InvoiceError, PR_PAID, PR_BROADCASTING, PR_BROADCA
 from electrum.logging import get_logger
 from electrum.network import TxBroadcastError, BestEffortRequestFailed
 from electrum.transaction import PartialTransaction, Transaction
-from electrum.util import InvalidPassword, event_listener, AddTransactionException, get_asyncio_loop
+from electrum.util import InvalidPassword, event_listener, AddTransactionException, get_asyncio_loop, NotEnoughFunds, \
+    NoDynamicFeeEstimates
+from electrum.lnutil import MIN_FUNDING_SAT
 from electrum.plugin import run_hook
 from electrum.wallet import Multisig_Wallet
 from electrum.crypto import pw_decode_with_version_and_mac
+from electrum.fee_policy import FeePolicy, FixedFeePolicy
 
 from .auth import AuthMixin, auth_protect
 from .qeaddresslistmodel import QEAddressCoinListModel
@@ -28,7 +31,7 @@ from .util import QtEventListener, qt_event_listener
 
 if TYPE_CHECKING:
     from electrum.wallet import Abstract_Wallet
-    from .qeinvoice import QEInvoice
+    from electrum.invoices import Invoice
 
 
 class QEWallet(AuthMixin, QObject, QtEventListener):
@@ -65,8 +68,6 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     paymentSucceeded = pyqtSignal([str], arguments=['key'])
     paymentFailed = pyqtSignal([str, str], arguments=['key', 'reason'])
     requestNewPassword = pyqtSignal()
-    signSucceeded = pyqtSignal([str], arguments=['txid'])
-    signFailed = pyqtSignal([str], arguments=['message'])
     broadcastSucceeded = pyqtSignal([str], arguments=['txid'])
     broadcastFailed = pyqtSignal([str, str, str], arguments=['txid', 'code', 'reason'])
     saveTxSuccess = pyqtSignal([str], arguments=['txid'])
@@ -101,10 +102,12 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
         self._frozenbalance = QEAmount()
         self._totalbalance = QEAmount()
         self._lightningcanreceive = QEAmount()
+        self._minchannelfunding = QEAmount(amount_sat=int(MIN_FUNDING_SAT))
         self._lightningcansend = QEAmount()
         self._lightningbalancefrozen = QEAmount()
 
         self._seed = ''
+        self._seed_passphrase = ''
 
         self.tx_notification_queue = queue.Queue()
         self.tx_notification_last_time = 0
@@ -161,6 +164,18 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
             self._logger.info(progress)
             self.synchronizingProgressChanged.emit()
 
+    multipleChangeChanged = pyqtSignal()
+    @pyqtProperty(bool, notify=multipleChangeChanged)
+    def multipleChange(self):
+        return self.wallet.multiple_change
+
+    @multipleChange.setter
+    def multipleChange(self, multiple_change):
+        if self.wallet.multiple_change != multiple_change:
+            self.wallet.multiple_change = multiple_change
+            self.wallet.db.put('multiple_change', self.wallet.multiple_change)
+            self.multipleChangeChanged.emit()
+
     @qt_event_listener
     def on_event_request_status(self, wallet, key, status):
         if wallet == self.wallet:
@@ -209,6 +224,7 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
         if wallet == self.wallet:
             self._logger.debug('wallet_updated')
             self.balanceChanged.emit()
+            self.historyModel.setDirty()
             self.synchronizing = not wallet.is_up_to_date()
             if not self.synchronizing:
                 self.historyModel.initModel()  # refresh if dirty
@@ -338,7 +354,9 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     billingInfoChanged = pyqtSignal()
     @pyqtProperty('QVariantMap', notify=billingInfoChanged)
     def billingInfo(self):
-        return {} if self.wallet.wallet_type != '2fa' else self.wallet.billing_info
+        if self.wallet.wallet_type != '2fa':
+            return {}
+        return self.wallet.billing_info if self.wallet.billing_info is not None else {}
 
     @pyqtProperty(bool, notify=dataChanged)
     def canHaveLightning(self):
@@ -360,6 +378,10 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     def seed(self):
         return self._seed
 
+    @pyqtProperty(str, notify=dataChanged)
+    def seedPassphrase(self):
+        return self._seed_passphrase
+
     @pyqtProperty(str, notify=dataChanged)
     def txinType(self):
         if self.wallet.wallet_type == 'imported':
@@ -440,6 +462,11 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     def canSignMessage(self):
         return not isinstance(self.wallet, Multisig_Wallet) and not self.wallet.is_watching_only()
 
+    canGetZeroconfChannelChanged = pyqtSignal()
+    @pyqtProperty(bool, notify=canGetZeroconfChannelChanged)
+    def canGetZeroconfChannel(self) -> bool:
+        return self.wallet.lnworker and self.wallet.lnworker.can_get_zeroconf_channel()
+
     @pyqtProperty(QEAmount, notify=balanceChanged)
     def frozenBalance(self):
         c, u, x = self.wallet.get_frozen_balance()
@@ -487,6 +514,14 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
             self._lightningcanreceive.satsInt = int(self.wallet.lnworker.num_sats_can_receive())
         return self._lightningcanreceive
 
+    @pyqtProperty(bool, notify=balanceChanged)
+    def isLowReserve(self):
+        return self.wallet.is_low_reserve()
+
+    @pyqtProperty(QEAmount, notify=dataChanged)
+    def minChannelFunding(self):
+        return self._minchannelfunding
+
     @pyqtProperty(int, notify=peersUpdated)
     def lightningNumPeers(self):
         if self.isLightning:
@@ -499,42 +534,46 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
         self.isLightningChanged.emit()
         self.dataChanged.emit()
 
-    @auth_protect()
-    def sign(self, tx, *, broadcast: bool = False, on_success: Callable[[Transaction], None] = None, on_failure: Callable[[], None] = None):
-        sign_hook = run_hook('tc_sign_wrapper', self.wallet, tx, partial(self.on_sign_complete, broadcast, on_success), partial(self.on_sign_failed, on_failure))
-        if sign_hook:
-            success = self.do_sign(tx, False)
-            if success:
-                self._logger.debug('plugin needs to sign tx too')
-                sign_hook(tx)
-                return
-        else:
-            success = self.do_sign(tx, broadcast)
+    @auth_protect(message=_('Sign and send on-chain transaction?'))
+    def sign_and_broadcast(self, tx, *,
+                           on_success: Callable[[Transaction], None] = None,
+                           on_failure: Callable[[Optional[Any]], None] = None) -> None:
+        self.do_sign(tx, True, on_success, on_failure)
 
-        if success:
-            if on_success:
-                on_success(tx)
-        else:
-            if on_failure:
-                on_failure()
+    @auth_protect(message=_('Sign on-chain transaction?'))
+    def sign(self, tx, *,
+             on_success: Callable[[Transaction], None] = None,
+             on_failure: Callable[[Optional[Any]], None] = None) -> None:
+        self.do_sign(tx, False, on_success, on_failure)
 
-    def do_sign(self, tx, broadcast):
+    def do_sign(self, tx, broadcast, on_success: Callable[[Transaction], None] = None, on_failure: Callable[[Optional[Any]], None] = None):
+        # tc_sign_wrapper is only used by 2fa. don't pass on_failure handler, it is handled via otpFailed signal
+        sign_hook = run_hook('tc_sign_wrapper', self.wallet, tx,
+                             partial(self.on_sign_complete, broadcast, on_success),
+                             partial(self.on_sign_failed, None))
         try:
             # ignore_warnings=True, because UI checks and asks user confirmation itself
             tx = self.wallet.sign_transaction(tx, self.password, ignore_warnings=True)
         except BaseException as e:
             self._logger.error(f'{e!r}')
-            self.signFailed.emit(str(e))
+            if on_failure:
+                on_failure(str(e))
+            return
 
         if tx is None:
             self._logger.info('did not sign')
-            return False
+            if on_failure:
+                on_failure()
+            return
+
+        if sign_hook:
+            self._logger.debug('plugin needs to sign tx too')
+            sign_hook(tx)
+            return
 
         txid = tx.txid()
         self._logger.debug(f'do_sign(), txid={txid}')
 
-        self.signSucceeded.emit(txid)
-
         if not tx.is_complete():
             self._logger.debug('tx not complete')
             broadcast = False
@@ -545,7 +584,8 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
             # not broadcasted, so refresh history here
             self.historyModel.initModel(True)
 
-        return True
+        if on_success:
+            on_success(tx)
 
     # this assumes a 2fa wallet, but there are no other tc_sign_wrapper hooks, so that's ok
     def on_sign_complete(self, broadcast, cb: Callable[[Transaction], None] = None, tx: Transaction = None):
@@ -617,12 +657,27 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
         self.paymentAuthRejected.emit()
 
     @auth_protect(message=_('Pay lightning invoice?'), reject='ln_auth_rejected')
-    def pay_lightning_invoice(self, invoice: 'QEInvoice'):
-        amount_msat = invoice.get_amount_msat()
+    def pay_lightning_invoice(self, invoice: 'Invoice', amount_msat: int = None):
+        # at this point, the user confirmed the payment, potentially with an override amount.
+        # we save the invoice with the override amount if there was no amount defined in the invoice.
+        # (this is similar to what the desktop client does)
+        #
+        # Note: amount_msat can be greater than the invoice-specified amount. This is validated and handled
+        # in lnworker.pay_invoice()
+        if amount_msat is not None:
+            assert type(amount_msat) is int
+            if invoice.get_amount_msat() is None:
+                invoice.set_amount_msat(amount_msat)
+        else:
+            amount_msat = invoice.get_amount_msat()
+
+        self.wallet.save_invoice(invoice)
+        if self._invoiceModel:
+            self._invoiceModel.initModel()
 
         def pay_thread():
             try:
-                coro = self.wallet.lnworker.pay_invoice(invoice.lightning_invoice, amount_msat=amount_msat)
+                coro = self.wallet.lnworker.pay_invoice(invoice, amount_msat=amount_msat)
                 fut = asyncio.run_coroutine_threadsafe(coro, get_asyncio_loop())
                 fut.result()
             except Exception as e:
@@ -641,24 +696,25 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     @pyqtSlot(QEAmount, str, int, bool)
     @pyqtSlot(QEAmount, str, int, bool, bool)
     @pyqtSlot(QEAmount, str, int, bool, bool, bool)
-    def createRequest(self, amount: QEAmount, message: str, expiration: int, lightning_only: bool = False, reuse_address: bool = False):
+    def createRequest(self, amount: QEAmount, message: str, expiration: int, lightning: bool = False, reuse_address: bool = False):
         self.deleteExpiredRequests()
         try:
             amount = amount.satsInt
-            addr = self.wallet.get_unused_address()
-            if addr is None:
-                if reuse_address:
-                    addr = self.wallet.get_receiving_address()
-                elif lightning_only:
-                    addr = None
-                else:
-                    msg = [
-                        _('No address available.'),
-                        _('All your addresses are used in pending requests.'),
-                        _('To see the list, press and hold the Receive button.'),
-                    ]
-                    self.requestCreateError.emit(' '.join(msg))
-                    return
+            if not lightning:
+                addr = self.wallet.get_unused_address()
+                if addr is None:
+                    if reuse_address:
+                        addr = self.wallet.get_receiving_address()
+                    else:
+                        msg = [
+                            _('No address available.'),
+                            _('All your addresses are used in pending requests.'),
+                            _('To see the list, press and hold the Receive button.'),
+                        ]
+                        self.requestCreateError.emit(' '.join(msg))
+                        return
+            else:
+                addr = None
 
             key = self.wallet.create_request(amount, message, expiration, addr)
         except InvoiceError as e:
@@ -709,12 +765,16 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
         try:
             self._logger.info('setting new password')
             self.wallet.update_password(current_password, password, encrypt_storage=True)
-            self.password = password
+            self.wallet.unlock(password)
             return True
         except InvalidPassword as e:
             self._logger.exception(repr(e))
             return False
 
+    @property
+    def password(self):
+        return self.wallet.get_unlocked_password()
+
     @pyqtSlot(str)
     def importAddresses(self, addresslist):
         self.wallet.import_addresses(addresslist.split())
@@ -756,9 +816,11 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     def retrieve_seed(self):
         try:
             self._seed = self.wallet.get_seed(self.password)
+            self._seed_passphrase = self.wallet.keystore.get_passphrase(self.password)
             self.seedRetrieved.emit()
         except Exception:
             self._seed = ''
+            self._seed_passphrase = ''
 
         self.dataChanged.emit()
 
@@ -770,15 +832,15 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
 
     @pyqtSlot(result='QVariantMap')
     def getBalancesForPiechart(self):
-        confirmed, unconfirmed, unmatured, frozen, lightning, f_lightning = balances = self.wallet.get_balances_for_piechart()
+        p_bal = self.wallet.get_balances_for_piechart()
         return {
-            'confirmed': confirmed,
-            'unconfirmed': unconfirmed,
-            'unmatured': unmatured,
-            'frozen': frozen,
-            'lightning': int(lightning),
-            'f_lightning': int(f_lightning),
-            'total': sum([int(x) for x in list(balances)])
+            'confirmed': p_bal.confirmed,
+            'unconfirmed': p_bal.unconfirmed,
+            'unmatured': p_bal.unmatured,
+            'frozen': p_bal.frozen,
+            'lightning': int(p_bal.lightning),
+            'f_lightning': int(p_bal.lightning_frozen),
+            'total': int(p_bal.total())
         }
 
     @pyqtSlot(str, result=bool)
@@ -789,3 +851,22 @@ class QEWallet(AuthMixin, QObject, QtEventListener):
     def signMessage(self, address, message):
         sig = self.wallet.sign_message(address, message, self.password)
         return base64.b64encode(sig).decode('ascii')
+
+    def determine_max(self, *, mktx: Callable[[FeePolicy], PartialTransaction]) -> Tuple[Optional[int], Optional[str]]:
+        # TODO: merge with SendTab.spend_max() and move to backend wallet
+        amount = message = None
+        try:
+            try:
+                fee_policy = FeePolicy(self.wallet.config.FEE_POLICY)
+                tx = mktx(fee_policy)
+            except (NotEnoughFunds, NoDynamicFeeEstimates) as e:
+                # Check if we had enough funds excluding fees,
+                # if so, still provide opportunity to set lower fees.
+                fee_policy = FixedFeePolicy(0)
+                tx = mktx(fee_policy)
+            amount = tx.output_value()
+        except NotEnoughFunds as e:
+            self._logger.debug(str(e))
+            message = self.wallet.get_text_not_enough_funds_mentioning_frozen(for_amount='!')
+
+        return amount, message
