@@ -24,7 +24,6 @@
 # SOFTWARE.
 
 import os
-import sys
 import time
 import datetime
 from datetime import date
@@ -33,20 +32,20 @@ import threading
 import enum
 from decimal import Decimal
 
-from PyQt5.QtGui import QFont, QBrush, QColor
-from PyQt5.QtCore import (Qt, QPersistentModelIndex, QModelIndex, QAbstractItemModel,
+from PyQt6.QtGui import QFont, QBrush, QColor
+from PyQt6.QtCore import (Qt, QPersistentModelIndex, QModelIndex,
                           QSortFilterProxyModel, QVariant, QItemSelectionModel, QDate, QPoint)
-from PyQt5.QtWidgets import (QMenu, QHeaderView, QLabel, QMessageBox,
-                             QPushButton, QComboBox, QVBoxLayout, QCalendarWidget,
+from PyQt6.QtWidgets import (QMenu, QHeaderView, QLabel, QPushButton, QComboBox, QVBoxLayout, QCalendarWidget,
                              QGridLayout)
 
 from electrum.gui import messages
-from electrum.address_synchronizer import TX_HEIGHT_LOCAL, TX_HEIGHT_FUTURE
+from electrum.address_synchronizer import TX_HEIGHT_LOCAL
 from electrum.i18n import _
 from electrum.util import (block_explorer_URL, profiler, TxMinedInfo,
                            OrderedDictWithIndex, timestamp_to_datetime,
-                           Satoshis, Fiat, format_time)
+                           Satoshis, format_time)
 from electrum.logging import get_logger, Logger
+from electrum.simple_config import SimpleConfig
 
 from .custom_model import CustomNode, CustomModel
 from .util import (read_QIcon, MONOSPACE_FONT, Buttons, CancelButton, OkButton,
@@ -62,7 +61,6 @@ if TYPE_CHECKING:
 _logger = get_logger(__name__)
 
 
-# note: this list needs to be kept in sync with another in kivy
 TX_ICONS = [
     "unconfirmed.png",
     "warning.png",
@@ -77,42 +75,80 @@ TX_ICONS = [
 ]
 
 
-ROLE_SORT_ORDER = Qt.UserRole + 1000
-
-
 class HistorySortModel(QSortFilterProxyModel):
+
+    def data_for(self, index: QModelIndex):
+        col = index.column()
+        if col == HistoryColumns.STATUS:
+            # respect sort order of self.transactions (wallet.get_full_history)
+            return index.row()
+        else:
+            node = index.internalPointer()
+            return node.sort_keys[col]
+
     def lessThan(self, source_left: QModelIndex, source_right: QModelIndex):
-        item1 = self.sourceModel().data(source_left, ROLE_SORT_ORDER)
-        item2 = self.sourceModel().data(source_right, ROLE_SORT_ORDER)
-        if item1 is None or item2 is None:
-            raise Exception(f'UserRole not set for column {source_left.column()}')
-        v1 = item1.value()
-        v2 = item2.value()
-        if v1 is None or isinstance(v1, Decimal) and v1.is_nan(): v1 = -float("inf")
-        if v2 is None or isinstance(v2, Decimal) and v2.is_nan(): v2 = -float("inf")
-        try:
-            return v1 < v2
-        except Exception:
-            return False
+        return self.data_for(source_left) < self.data_for(source_right)
+
 
 def get_item_key(tx_item):
     return tx_item.get('txid') or tx_item['payment_hash']
 
+def flatten_sort_key(v):
+    if v is None or isinstance(v, Decimal) and v.is_nan():
+        return -float("inf")
+    else:
+        return v
+
 
 class HistoryNode(CustomNode):
 
     model: 'HistoryModel'
 
+    def __init__(self, model: 'CustomModel', tx_item):
+        super().__init__(model, tx_item)
+
+        if tx_item is None:
+            tx_item = {}
+        is_lightning = tx_item.get('lightning', False)
+        short_id = ""
+        if not is_lightning:
+            txpos_in_block = tx_item.get('txpos_in_block') or -1
+            if txpos_in_block >= 0:
+                short_id = f"{tx_item['height']}x{txpos_in_block}"
+        self.sort_keys = {
+            HistoryColumns.DESCRIPTION: flatten_sort_key(
+                tx_item.get('label')),
+            HistoryColumns.AMOUNT: flatten_sort_key(
+                (tx_item['bc_value'].value if 'bc_value' in tx_item else 0)\
+                    + (tx_item['ln_value'].value if 'ln_value' in tx_item else 0)),
+            HistoryColumns.BALANCE: 0,
+            HistoryColumns.FIAT_VALUE: flatten_sort_key(
+                tx_item['fiat_value'].value if 'fiat_value' in tx_item else None),
+            HistoryColumns.FIAT_ACQ_PRICE: flatten_sort_key(
+                tx_item['acquisition_price'].value if 'acquisition_price' in tx_item else None),
+            HistoryColumns.FIAT_CAP_GAINS: flatten_sort_key(
+                tx_item['capital_gain'].value if 'capital_gain' in tx_item else None),
+            HistoryColumns.TXID: flatten_sort_key(
+                tx_item.get('txid') if not is_lightning else None),
+            HistoryColumns.SHORT_ID:
+                short_id,
+        }
+
+    def set_balance(self, balance):
+        self._data['balance'] = Satoshis(balance)
+        self.sort_keys[HistoryColumns.BALANCE] = balance
+
     def get_data_for_role(self, index: QModelIndex, role: Qt.ItemDataRole) -> QVariant:
-        # note: this method is performance-critical.
-        # it is called a lot, and so must run extremely fast.
         assert index.isValid()
         col = index.column()
         window = self.model.window
         tx_item = self.get_data()
         is_lightning = tx_item.get('lightning', False)
+        if not is_lightning and 'txid' not in tx_item:
+            # this may happen if two lightning tx have the same group id
+            # and the group does not have an onchain tx
+            is_lightning = True
         timestamp = tx_item['timestamp']
-        short_id = None
         if is_lightning:
             status = 0
             if timestamp is None:
@@ -121,9 +157,6 @@ class HistoryNode(CustomNode):
                 status_str = format_time(int(timestamp))
         else:
             tx_hash = tx_item['txid']
-            txpos_in_block = tx_item.get('txpos_in_block')
-            if txpos_in_block is not None and txpos_in_block >= 0:
-                short_id = f"{tx_item['height']}x{txpos_in_block}"
             conf = tx_item['confirmations']
             try:
                 status, status_str = self.model.tx_status_cache[tx_hash]
@@ -131,35 +164,13 @@ class HistoryNode(CustomNode):
                 tx_mined_info = self.model._tx_mined_info_from_tx_item(tx_item)
                 status, status_str = window.wallet.get_tx_status(tx_hash, tx_mined_info)
 
-        if role == ROLE_SORT_ORDER:
-            d = {
-                HistoryColumns.STATUS:
-                    # respect sort order of self.transactions (wallet.get_full_history)
-                    -index.row(),
-                HistoryColumns.DESCRIPTION:
-                    tx_item['label'] if 'label' in tx_item else None,
-                HistoryColumns.AMOUNT:
-                    (tx_item['bc_value'].value if 'bc_value' in tx_item else 0)\
-                    + (tx_item['ln_value'].value if 'ln_value' in tx_item else 0),
-                HistoryColumns.BALANCE:
-                    (tx_item['balance'].value if 'balance' in tx_item else 0),
-                HistoryColumns.FIAT_VALUE:
-                    tx_item['fiat_value'].value if 'fiat_value' in tx_item else None,
-                HistoryColumns.FIAT_ACQ_PRICE:
-                    tx_item['acquisition_price'].value if 'acquisition_price' in tx_item else None,
-                HistoryColumns.FIAT_CAP_GAINS:
-                    tx_item['capital_gain'].value if 'capital_gain' in tx_item else None,
-                HistoryColumns.TXID: tx_hash if not is_lightning else None,
-                HistoryColumns.SHORT_ID: short_id,
-            }
-            return QVariant(d[col])
         if role == MyTreeView.ROLE_EDIT_KEY:
             return QVariant(get_item_key(tx_item))
-        if role not in (Qt.DisplayRole, Qt.EditRole):
-            if col == HistoryColumns.STATUS and role == Qt.DecorationRole:
+        if role not in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole, MyTreeView.ROLE_CLIPBOARD_DATA):
+            if col == HistoryColumns.STATUS and role == Qt.ItemDataRole.DecorationRole:
                 icon = "lightning" if is_lightning else TX_ICONS[status]
                 return QVariant(read_QIcon(icon))
-            elif col == HistoryColumns.STATUS and role == Qt.ToolTipRole:
+            elif col == HistoryColumns.STATUS and role == Qt.ItemDataRole.ToolTipRole:
                 if is_lightning:
                     msg = 'lightning transaction'
                 else:  # on-chain
@@ -171,23 +182,30 @@ class HistoryNode(CustomNode):
                     else:
                         msg = str(conf) + _(" confirmation" + ("s" if conf != 1 else ""))
                 return QVariant(msg)
-            elif col > HistoryColumns.DESCRIPTION and role == Qt.TextAlignmentRole:
-                return QVariant(int(Qt.AlignRight | Qt.AlignVCenter))
-            elif col > HistoryColumns.DESCRIPTION and role == Qt.FontRole:
+            elif col > HistoryColumns.DESCRIPTION and role == Qt.ItemDataRole.TextAlignmentRole:
+                return QVariant(int(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter))
+            elif col > HistoryColumns.DESCRIPTION and role == Qt.ItemDataRole.FontRole:
                 monospace_font = QFont(MONOSPACE_FONT)
                 return QVariant(monospace_font)
-            #elif col == HistoryColumns.DESCRIPTION and role == Qt.DecorationRole and not is_lightning\
+            #elif col == HistoryColumns.DESCRIPTION and role == Qt.ItemDataRole.DecorationRole and not is_lightning\
             #        and self.parent.wallet.invoices.paid.get(tx_hash):
             #    return QVariant(read_QIcon("seal"))
             elif col in (HistoryColumns.DESCRIPTION, HistoryColumns.AMOUNT) \
-                    and role == Qt.ForegroundRole and tx_item['value'].value < 0:
+                    and role == Qt.ItemDataRole.ForegroundRole and tx_item['value'].value < 0:
                 red_brush = QBrush(QColor("#BC1E1E"))
                 return QVariant(red_brush)
-            elif col == HistoryColumns.FIAT_VALUE and role == Qt.ForegroundRole \
+            elif col == HistoryColumns.FIAT_VALUE and role == Qt.ItemDataRole.ForegroundRole \
                     and not tx_item.get('fiat_default') and tx_item.get('fiat_value') is not None:
                 blue_brush = QBrush(QColor("#1E1EFF"))
                 return QVariant(blue_brush)
             return QVariant()
+
+        add_thousands_sep = None
+        whitespaces = True
+        if role == MyTreeView.ROLE_CLIPBOARD_DATA:
+            add_thousands_sep = False
+            whitespaces = False
+
         if col == HistoryColumns.STATUS:
             return QVariant(status_str)
         elif col == HistoryColumns.DESCRIPTION and 'label' in tx_item:
@@ -196,27 +214,27 @@ class HistoryNode(CustomNode):
             bc_value = tx_item['bc_value'].value if 'bc_value' in tx_item else 0
             ln_value = tx_item['ln_value'].value if 'ln_value' in tx_item else 0
             value = bc_value + ln_value
-            v_str = window.format_amount(value, is_diff=True, whitespaces=True)
+            v_str = window.format_amount(value, is_diff=True, whitespaces=whitespaces, add_thousands_sep=add_thousands_sep)
             return QVariant(v_str)
         elif col == HistoryColumns.BALANCE:
-            balance = tx_item['balance'].value
-            balance_str = window.format_amount(balance, whitespaces=True)
+            balance = tx_item['balance'].value if 'balance' in tx_item else None
+            balance_str = window.format_amount(balance, whitespaces=whitespaces, add_thousands_sep=add_thousands_sep) if balance is not None else ''
             return QVariant(balance_str)
         elif col == HistoryColumns.FIAT_VALUE and 'fiat_value' in tx_item:
-            value_str = window.fx.format_fiat(tx_item['fiat_value'].value)
+            value_str = window.fx.format_fiat(tx_item['fiat_value'].value, add_thousands_sep=add_thousands_sep)
             return QVariant(value_str)
         elif col == HistoryColumns.FIAT_ACQ_PRICE and \
                 tx_item['value'].value < 0 and 'acquisition_price' in tx_item:
             # fixme: should use is_mine
             acq = tx_item['acquisition_price'].value
-            return QVariant(window.fx.format_fiat(acq))
+            return QVariant(window.fx.format_fiat(acq, add_thousands_sep=add_thousands_sep))
         elif col == HistoryColumns.FIAT_CAP_GAINS and 'capital_gain' in tx_item:
             cg = tx_item['capital_gain'].value
-            return QVariant(window.fx.format_fiat(cg))
+            return QVariant(window.fx.format_fiat(cg, add_thousands_sep=add_thousands_sep))
         elif col == HistoryColumns.TXID:
             return QVariant(tx_hash) if not is_lightning else QVariant('')
         elif col == HistoryColumns.SHORT_ID:
-            return QVariant(short_id or "")
+            return QVariant(self.sort_keys[HistoryColumns.SHORT_ID])
         return QVariant()
 
 
@@ -240,19 +258,19 @@ class HistoryModel(CustomModel, Logger):
         tx_item = index.internalPointer().get_data()
         tx_item['label'] = self.window.wallet.get_label_for_txid(get_item_key(tx_item))
         topLeft = bottomRight = self.createIndex(index.row(), HistoryColumns.DESCRIPTION)
-        self.dataChanged.emit(topLeft, bottomRight, [Qt.DisplayRole])
+        self.dataChanged.emit(topLeft, bottomRight, [Qt.ItemDataRole.DisplayRole])
         self.window.utxo_list.update()
 
     def get_domain(self):
         """Overridden in address_dialog.py"""
-        return self.window.wallet.get_addresses()
+        return None
 
     def should_include_lightning_payments(self) -> bool:
         """Overridden in address_dialog.py"""
         return True
 
     def should_show_fiat(self):
-        if not bool(self.window.config.get('history_rates', False)):
+        if not self.window.config.FX_HISTORY_RATES:
             return False
         fx = self.window.fx
         if not fx or not fx.is_enabled():
@@ -260,7 +278,7 @@ class HistoryModel(CustomModel, Logger):
         return fx.has_history()
 
     def should_show_capital_gains(self):
-        return self.should_show_fiat() and self.window.config.get('history_rates_capital_gains', False)
+        return self.should_show_fiat() and self.window.config.FX_HISTORY_RATES_CAPITAL_GAINS
 
     @profiler
     def refresh(self, reason: str):
@@ -274,7 +292,8 @@ class HistoryModel(CustomModel, Logger):
         if selected:
             selected_row = selected.row()
         fx = self.window.fx
-        if fx: fx.history_used_spot = False
+        if fx:
+            fx.history_used_spot = False
         wallet = self.window.wallet
         self.set_visibility_of_columns()
         transactions = wallet.get_full_history(
@@ -283,8 +302,6 @@ class HistoryModel(CustomModel, Logger):
             include_lightning=self.should_include_lightning_payments(),
             include_fiat=self.should_show_fiat(),
         )
-        if transactions == self.transactions:
-            return
         old_length = self._root.childCount()
         if old_length != 0:
             self.beginRemoveRows(QModelIndex(), 0, old_length)
@@ -294,43 +311,24 @@ class HistoryModel(CustomModel, Logger):
         parents = {}
         for tx_item in transactions.values():
             node = HistoryNode(self, tx_item)
-            group_id = tx_item.get('group_id')
-            if group_id is None:
-                self._root.addChild(node)
-            else:
-                parent = parents.get(group_id)
-                if parent is None:
-                    # create parent if it does not exist
-                    self._root.addChild(node)
-                    parents[group_id] = node
-                else:
-                    # if parent has no children, create two children
-                    if parent.childCount() == 0:
-                        child_data = dict(parent.get_data())
-                        node1 = HistoryNode(self, child_data)
-                        parent.addChild(node1)
-                        parent._data['label'] = child_data.get('group_label')
-                        parent._data['bc_value'] = child_data.get('bc_value', Satoshis(0))
-                        parent._data['ln_value'] = child_data.get('ln_value', Satoshis(0))
-                    # add child to parent
-                    parent.addChild(node)
-                    # update parent data
-                    parent._data['balance'] = tx_item['balance']
-                    parent._data['value'] += tx_item['value']
-                    if 'group_label' in tx_item:
-                        parent._data['label'] = tx_item['group_label']
-                    if 'bc_value' in tx_item:
-                        parent._data['bc_value'] += tx_item['bc_value']
-                    if 'ln_value' in tx_item:
-                        parent._data['ln_value'] += tx_item['ln_value']
-                    if 'fiat_value' in tx_item:
-                        parent._data['fiat_value'] += tx_item['fiat_value']
-                    if tx_item.get('txid') == group_id:
-                        parent._data['lightning'] = False
-                        parent._data['txid'] = tx_item['txid']
-                        parent._data['timestamp'] = tx_item['timestamp']
-                        parent._data['height'] = tx_item['height']
-                        parent._data['confirmations'] = tx_item['confirmations']
+            self._root.addChild(node)
+            for child_item in tx_item.get('children', []):
+                child_node = HistoryNode(self, child_item)
+                # add child to parent
+                node.addChild(child_node)
+
+        # compute balance once all children have been added
+        balance = 0
+        for node in self._root._children:
+            balance += node._data['value'].value
+            node.set_balance(balance)
+
+        # update tx_status_cache  (before endInsertRows() triggers get_data_for_role() calls)
+        self.tx_status_cache.clear()
+        for txid, tx_item in transactions.items():
+            if not tx_item.get('lightning', False):
+                tx_mined_info = self._tx_mined_info_from_tx_item(tx_item)
+                self.tx_status_cache[txid] = self.window.wallet.get_tx_status(txid, tx_mined_info)
 
         new_length = self._root.childCount()
         self.beginInsertRows(QModelIndex(), 0, new_length-1)
@@ -338,7 +336,9 @@ class HistoryModel(CustomModel, Logger):
         self.endInsertRows()
 
         if selected_row:
-            self.view.selectionModel().select(self.createIndex(selected_row, 0), QItemSelectionModel.Rows | QItemSelectionModel.SelectCurrent)
+            self.view.selectionModel().select(
+                self.createIndex(selected_row, 0),
+                QItemSelectionModel.SelectionFlag.Rows | QItemSelectionModel.SelectionFlag.SelectCurrent)
         self.view.filter()
         # update time filter
         if not self.view.years and self.transactions:
@@ -349,12 +349,6 @@ class HistoryModel(CustomModel, Logger):
                 end_date = self.transactions.value_from_pos(len(self.transactions) - 1).get('date') or end_date
             self.view.years = [str(i) for i in range(start_date.year, end_date.year + 1)]
             self.view.period_combo.insertItems(1, self.view.years)
-        # update tx_status_cache
-        self.tx_status_cache.clear()
-        for txid, tx_item in self.transactions.items():
-            if not tx_item.get('lightning', False):
-                tx_mined_info = self._tx_mined_info_from_tx_item(tx_item)
-                self.tx_status_cache[txid] = self.window.wallet.get_tx_status(txid, tx_mined_info)
         # update counter
         num_tx = len(self.transactions)
         if self.view:
@@ -363,6 +357,7 @@ class HistoryModel(CustomModel, Logger):
     def set_visibility_of_columns(self):
         def set_visible(col: int, b: bool):
             self.view.showColumn(col) if b else self.view.hideColumn(col)
+
         # txid
         set_visible(HistoryColumns.TXID, False)
         set_visible(HistoryColumns.SHORT_ID, False)
@@ -381,7 +376,7 @@ class HistoryModel(CustomModel, Logger):
         fiat_fields = self.window.wallet.get_tx_item_fiat(
             tx_hash=txid, amount_sat=value, fx=self.window.fx, tx_fee=fee.value if fee else None)
         tx_item.update(fiat_fields)
-        self.dataChanged.emit(idx, idx, [Qt.DisplayRole, Qt.ForegroundRole])
+        self.dataChanged.emit(idx, idx, [Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.ForegroundRole])
 
     def update_tx_mined_status(self, tx_hash: str, tx_mined_info: TxMinedInfo):
         try:
@@ -410,18 +405,18 @@ class HistoryModel(CustomModel, Logger):
                 continue
             self.update_tx_mined_status(tx_hash, tx_mined_info)
 
-    def headerData(self, section: int, orientation: Qt.Orientation, role: Qt.ItemDataRole):
-        assert orientation == Qt.Horizontal
-        if role != Qt.DisplayRole:
+    def headerData(self, section: int, orientation: Qt.Orientation, role: int = Qt.ItemDataRole.DisplayRole):
+        assert orientation == Qt.Orientation.Horizontal
+        if role != Qt.ItemDataRole.DisplayRole:
             return None
         fx = self.window.fx
         fiat_title = 'n/a fiat value'
         fiat_acq_title = 'n/a fiat acquisition price'
         fiat_cg_title = 'n/a fiat capital gains'
         if self.should_show_fiat():
-            fiat_title = '%s '%fx.ccy + _('Value')
-            fiat_acq_title = '%s '%fx.ccy + _('Acquisition price')
-            fiat_cg_title =  '%s '%fx.ccy + _('Capital Gains')
+            fiat_title = '%s ' % fx.ccy + _('Value')
+            fiat_acq_title = '%s ' % fx.ccy + _('Acquisition price')
+            fiat_cg_title = '%s ' % fx.ccy + _('Capital Gains')
         return {
             HistoryColumns.STATUS: _('Date'),
             HistoryColumns.DESCRIPTION: _('Description'),
@@ -434,11 +429,11 @@ class HistoryModel(CustomModel, Logger):
             HistoryColumns.SHORT_ID: 'Short ID',
         }[section]
 
-    def flags(self, idx: QModelIndex) -> int:
-        extra_flags = Qt.NoItemFlags  # type: Qt.ItemFlag
+    def flags(self, idx: QModelIndex) -> Qt.ItemFlag:
+        extra_flags = Qt.ItemFlag.NoItemFlags  # type: Qt.ItemFlag
         if idx.column() in self.view.editable_columns:
-            extra_flags |= Qt.ItemIsEditable
-        return super().flags(idx) | int(extra_flags)
+            extra_flags |= Qt.ItemFlag.ItemIsEditable
+        return super().flags(idx) | extra_flags
 
     @staticmethod
     def _tx_mined_info_from_tx_item(tx_item: Dict[str, Any]) -> TxMinedInfo:
@@ -512,12 +507,14 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         self.period_combo.addItems([_('All'), _('Custom')])
         self.period_combo.activated.connect(self.on_combo)
         self.wallet = self.main_window.wallet  # type: Abstract_Wallet
-        self.sortByColumn(HistoryColumns.STATUS, Qt.AscendingOrder)
+        self.sortByColumn(HistoryColumns.STATUS, Qt.SortOrder.DescendingOrder)
         self.setRootIsDecorated(True)
         self.header().setStretchLastSection(False)
         for col in HistoryColumns:
-            sm = QHeaderView.Stretch if col == self.stretch_column else QHeaderView.ResizeToContents
+            sm = QHeaderView.ResizeMode.Stretch if col == self.stretch_column else QHeaderView.ResizeMode.ResizeToContents
             self.header().setSectionResizeMode(col, sm)
+        if self.config:
+            self.configvar_show_toolbar = self.config.cv.GUI_QT_HISTORY_TAB_SHOW_TOOLBAR
 
     def update(self):
         self.hm.refresh('HistoryList.update()')
@@ -546,13 +543,12 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
             self.end_button.setText(_('To') + ' ' + self.format_date(self.end_date))
         self.hide_rows()
 
-    CONFIG_KEY_SHOW_TOOLBAR = "show_toolbar_history"
-    def create_toolbar(self, config):
+    def create_toolbar(self, config: 'SimpleConfig'):
         toolbar, menu = self.create_toolbar_with_menu('')
         self.num_tx_label = toolbar.itemAt(0).widget()
         self._toolbar_checkbox = menu.addToggle(_("Filter by Date"), lambda: self.toggle_toolbar())
-        self.menu_fiat = menu.addConfig(_('Show Fiat Values'), 'history_rates', False, callback=self.main_window.app.update_fiat_signal.emit)
-        self.menu_capgains = menu.addConfig(_('Show Capital Gains'), 'history_rates_capital_gains', False, callback=self.main_window.app.update_fiat_signal.emit)
+        self.menu_fiat = menu.addConfig(config.cv.FX_HISTORY_RATES, short_desc=_('Show Fiat Values'), callback=self.main_window.app.update_fiat_signal.emit)
+        self.menu_capgains = menu.addConfig(config.cv.FX_HISTORY_RATES_CAPITAL_GAINS, callback=self.main_window.app.update_fiat_signal.emit)
         self.menu_summary = menu.addAction(_("&Summary"), self.show_summary)
         menu.addAction(_("&Plot"), self.plot_history_dialog)
         menu.addAction(_("&Export"), self.export_history_dialog)
@@ -590,15 +586,17 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         d.setMinimumSize(600, 150)
         d.date = None
         vbox = QVBoxLayout()
+
         def on_date(date):
             d.date = date
+
         cal = QCalendarWidget()
         cal.setGridVisible(True)
         cal.clicked[QDate].connect(on_date)
         vbox.addWidget(cal)
         vbox.addLayout(Buttons(OkButton(d), CancelButton(d)))
         d.setLayout(vbox)
-        if d.exec_():
+        if d.exec():
             if d.date is None:
                 return None
             date = d.date.toPyDate()
@@ -610,11 +608,10 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
             self.main_window.show_message(_("Enable fiat exchange rate with history."))
             return
         fx = self.main_window.fx
-        h = self.wallet.get_detailed_history(
-            from_timestamp = time.mktime(self.start_date.timetuple()) if self.start_date else None,
-            to_timestamp = time.mktime(self.end_date.timetuple()) if self.end_date else None,
+        summary = self.wallet.get_onchain_capital_gains(
+            from_timestamp=time.mktime(self.start_date.timetuple()) if self.start_date else None,
+            to_timestamp=time.mktime(self.end_date.timetuple()) if self.end_date else None,
             fx=fx)
-        summary = h['summary']
         if not summary:
             self.main_window.show_message(_("Nothing to summarize."))
             return
@@ -676,18 +673,20 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         vbox.addLayout(grid2)
         vbox.addLayout(Buttons(CloseButton(d)))
         d.setLayout(vbox)
-        d.exec_()
+        d.exec()
 
     def plot_history_dialog(self):
         try:
             from electrum.plot import plot_history, NothingToPlotException
-        except Exception as e:
+        except ImportError as e:
             _logger.error(f"could not import electrum.plot. This feature needs matplotlib to be installed. exc={e!r}")
-            self.main_window.show_message(
-                _("Can't plot history.") + '\n' +
-                _("Perhaps some dependencies are missing...") + " (matplotlib?)" + '\n' +
-                f"Error: {e!r}"
-            )
+            self.main_window.show_message("\n\n".join([
+                _("This feature requires the 'matplotlib' Python library which is not "
+                  "included in Electrum by default."),
+                _("If you run Electrum from source you can install matplotlib to use this feature."),
+                _("It is not possible to install matplotlib inside the binary executables "
+                  "(e.g. AppImage or Windows installation).")
+            ]))
             return
         try:
             plt = plot_history(list(self.hm.transactions.values()))
@@ -701,7 +700,7 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         column = index.column()
         key = get_item_key(tx_item)
         if column == HistoryColumns.DESCRIPTION:
-            if self.wallet.set_label(key, text): #changed
+            if self.wallet.set_label(key, text):  # changed
                 self.hm.update_label(index)
                 self.main_window.update_completions()
         elif column == HistoryColumns.FIAT_VALUE:
@@ -729,12 +728,14 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         for column in HistoryColumns:
             if self.isColumnHidden(column):
                 continue
-            column_title = self.hm.headerData(column, Qt.Horizontal, Qt.DisplayRole)
+            column_title = self.hm.headerData(column, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole)
             idx2 = idx.sibling(idx.row(), column)
-            column_data = (self.hm.data(idx2, Qt.DisplayRole).value() or '').strip()
+            clipboard_data = self.hm.data(idx2, self.ROLE_CLIPBOARD_DATA).value()
+            if clipboard_data is None:
+                clipboard_data = (self.hm.data(idx2, Qt.ItemDataRole.DisplayRole).value() or '').strip()
             cc.addAction(
                 column_title,
-                lambda text=column_data, title=column_title:
+                lambda text=clipboard_data, title=column_title:
                 self.place_text_on_clipboard(text, title=title))
         return cc
 
@@ -745,7 +746,7 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
             # can happen e.g. before list is populated for the first time
             return
         tx_item = idx.internalPointer().get_data()
-        if tx_item.get('lightning') and tx_item['type'] == 'payment':
+        if tx_item.get('lightning'):
             menu = QMenu()
             menu.addAction(_("Details"), lambda: self.main_window.show_lightning_transaction(tx_item))
             cc = self.add_copy_menu(menu, idx)
@@ -755,7 +756,7 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
             log = self.wallet.lnworker.logs.get(key)
             if log:
                 menu.addAction(_("View log"), lambda: self.main_window.send_tab.invoice_list.show_log(key, log))
-            menu.exec_(self.viewport().mapToGlobal(position))
+            menu.exec(self.viewport().mapToGlobal(position))
             return
         tx_hash = tx_item['txid']
         tx = self.wallet.adb.get_transaction(tx_hash)
@@ -772,8 +773,9 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         copy_menu.addAction(_("Transaction ID"), lambda: self.place_text_on_clipboard(tx_hash, title="TXID"))
         menu_edit = menu.addMenu(_("Edit"))
         for c in self.editable_columns:
-            if self.isColumnHidden(c): continue
-            label = self.hm.headerData(c, Qt.Horizontal, Qt.DisplayRole)
+            if self.isColumnHidden(c):
+                continue
+            label = self.hm.headerData(c, Qt.Orientation.Horizontal, Qt.ItemDataRole.DisplayRole)
             # TODO use siblingAtColumn when min Qt version is >=5.11
             persistent = QPersistentModelIndex(org_idx.sibling(org_idx.row(), c))
             menu_edit.addAction(_("{}").format(label), lambda p=persistent: self.edit(QModelIndex(p)))
@@ -797,7 +799,7 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
                 menu_invs.addAction(_("View invoice"), lambda inv=inv: self.main_window.show_onchain_invoice(inv))
         if tx_URL:
             menu.addAction(_("View on block explorer"), lambda: webopen(tx_URL))
-        menu.exec_(self.viewport().mapToGlobal(position))
+        menu.exec(self.viewport().mapToGlobal(position))
 
     def remove_local_tx(self, tx_hash: str):
         num_child_txs = len(self.wallet.adb.get_depending_transactions(tx_hash))
@@ -805,8 +807,7 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         if num_child_txs > 0:
             question = (_("Are you sure you want to remove this transaction and {} child transactions?")
                         .format(num_child_txs))
-        if not self.main_window.question(msg=question,
-                                    title=_("Please confirm")):
+        if not self.main_window.question(msg=question, title=_("Please confirm")):
             return
         self.wallet.adb.remove_transaction(tx_hash)
         self.wallet.save_db()
@@ -837,7 +838,7 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         vbox.addLayout(hbox)
         #run_hook('export_history_dialog', self, hbox)
         self.update()
-        if not d.exec_():
+        if not d.exec():
             return
         filename = filename_e.text()
         if not filename:
@@ -851,29 +852,52 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
         self.main_window.show_message(_("Your wallet history has been successfully exported."))
 
     def do_export_history(self, file_name, is_csv):
-        hist = self.wallet.get_detailed_history(fx=self.main_window.fx)
-        txns = hist['transactions']
+        txns = self.wallet.get_full_history(fx=self.main_window.fx)
         lines = []
+
+        def get_all_fees_paid_sat(h_item: dict) -> int:
+            # gets all fees paid in an item (or group), as the outer group doesn't contain the
+            # transaction fees paid by the children
+            fees_sat = 0
+            for child in h_item.get('children', []):
+                fees_sat += child['fee_sat'] or 0 if 'fee_sat' in child \
+                                else (child.get('fee_msat', 0) or 0) // 1000
+
+            fees_sat += h_item['fee_sat'] or 0 if 'fee_sat' in h_item \
+                            else (h_item.get('fee_msat', 0) or 0) // 1000
+            return fees_sat
+
         if is_csv:
-            for item in txns:
-                lines.append([item['txid'],
-                              item.get('label', ''),
-                              item['confirmations'],
-                              item['bc_value'],
-                              item.get('fiat_value', ''),
-                              item.get('fee', ''),
-                              item.get('fiat_fee', ''),
-                              item['date']])
+            # sort by timestamp so the generated csv is more understandable on first sight
+            txns = dict(sorted(txns.items(), key=lambda h_item: h_item[1]['timestamp'] or 0))
+            for item in txns.values():
+                # tx groups will are shown as single element
+                line = [
+                    item.get('txid', ''),
+                    item.get('payment_hash', ''),
+                    item.get('label', ''),
+                    item.get('confirmations', ''),
+                    item['bc_value'],
+                    item['ln_value'],
+                    item.get('fiat_value', ''),
+                    get_all_fees_paid_sat(item),
+                    item.get('fiat_fee', ''),
+                    item['date']
+                ]
+                lines.append(line)
+
         with open(file_name, "w+", encoding='utf-8') as f:
             if is_csv:
                 import csv
                 transaction = csv.writer(f, lineterminator='\n')
-                transaction.writerow(["transaction_hash",
+                transaction.writerow(["oc_transaction_hash",
+                                      "ln_payment_hash",
                                       "label",
                                       "confirmations",
-                                      "value",
+                                      "amount_chain_bc",
+                                      "amount_lightning_bc",
                                       "fiat_value",
-                                      "fee",
+                                      "network_fee_satoshi",
                                       "fiat_fee",
                                       "timestamp"])
                 for line in lines:
@@ -883,7 +907,7 @@ class HistoryList(MyTreeView, AcceptFileDragDrop):
                 f.write(json_encode(txns))
 
     def get_text_from_coordinate(self, row, col):
-        return self.get_role_data_from_coordinate(row, col, role=Qt.DisplayRole)
+        return self.get_role_data_from_coordinate(row, col, role=Qt.ItemDataRole.DisplayRole)
 
     def get_role_data_from_coordinate(self, row, col, *, role):
         idx = self.model().mapToSource(self.model().index(row, col))
