@@ -32,7 +32,10 @@ from typing import Tuple, TYPE_CHECKING, Union, Sequence, Optional, Dict, List,
 from functools import lru_cache, wraps
 from abc import ABC, abstractmethod
 
-from . import bitcoin, ecc, constants, bip32
+import electrum_ecc as ecc
+from electrum_ecc import string_to_number
+
+from . import bitcoin, constants, bip32
 from .bitcoin import deserialize_privkey, serialize_privkey, BaseDecodeError
 from .transaction import Transaction, PartialTransaction, PartialTxInput, PartialTxOutput, TxInput
 from .bip32 import (convert_bip32_strpath_to_intpath, BIP32_PRIME,
@@ -40,25 +43,24 @@ from .bip32 import (convert_bip32_strpath_to_intpath, BIP32_PRIME,
                     convert_bip32_intpath_to_strpath, is_xkey_consistent_with_key_origin_info,
                     KeyOriginInfo)
 from .descriptor import PubkeyProvider
-from .ecc import string_to_number
+from . import crypto
 from .crypto import (pw_decode, pw_encode, sha256, sha256d, PW_HASH_VERSION_LATEST,
                      SUPPORTED_PW_HASH_VERSIONS, UnsupportedPasswordHashVersion, hash_160,
                      CiphertextFormatError)
 from .util import (InvalidPassword, WalletFileException,
                    BitcoinException, bfh, inv_dict, is_hex_str)
-from .mnemonic import Mnemonic, Wordlist, seed_type, is_seed
+from .mnemonic import Mnemonic, Wordlist, calc_seed_type, is_seed
 from .plugin import run_hook
 from .logging import Logger
 
 if TYPE_CHECKING:
-    from .gui.qt.util import TaskThread
-    from .plugins.hw_wallet import HW_PluginBase, HardwareClientBase, HardwareHandlerBase
+    from .gui.common_qt.util import TaskThread
+    from .hw_wallet import HW_PluginBase, HardwareClientBase, HardwareHandlerBase
     from .wallet_db import WalletDB
     from .plugin import Device
 
 
 class CannotDerivePubkey(Exception): pass
-
 class ScriptTypeNotSupported(Exception): pass
 
 
@@ -107,24 +109,24 @@ class KeyStore(Logger, ABC):
         """Returns whether the keystore can be encrypted with a password."""
         pass
 
-    def _get_tx_derivations(self, tx: 'PartialTransaction') -> Dict[str, Union[Sequence[int], str]]:
+    def _get_tx_derivations(self, tx: 'PartialTransaction') -> Dict[bytes, Union[Sequence[int], str]]:
         keypairs = {}
         for txin in tx.inputs():
             keypairs.update(self._get_txin_derivations(txin))
         return keypairs
 
-    def _get_txin_derivations(self, txin: 'PartialTxInput') -> Dict[str, Union[Sequence[int], str]]:
+    def _get_txin_derivations(self, txin: 'PartialTxInput') -> Dict[bytes, Union[Sequence[int], str]]:
         if txin.is_complete():
             return {}
         keypairs = {}
         for pubkey in txin.pubkeys:
-            if pubkey in txin.part_sigs:
+            if pubkey in txin.sigs_ecdsa:
                 # this pubkey already signed
                 continue
             derivation = self.get_pubkey_derivation(pubkey, txin)
             if not derivation:
                 continue
-            keypairs[pubkey.hex()] = derivation
+            keypairs[pubkey] = derivation
         return keypairs
 
     def can_sign(self, tx: 'Transaction', *, ignore_watching_only=False) -> bool:
@@ -202,6 +204,12 @@ class KeyStore(Logger, ABC):
     def can_have_deterministic_lightning_xprv(self) -> bool:
         return False
 
+    def has_support_for_slip_19_ownership_proofs(self) -> bool:
+        return False
+
+    def add_slip_19_ownership_proofs_to_tx(self, tx: 'PartialTransaction', *, password) -> None:
+        raise NotImplementedError()
+
 
 class Software_KeyStore(KeyStore):
 
@@ -217,12 +225,12 @@ class Software_KeyStore(KeyStore):
     def sign_message(self, sequence, message, password, *, script_type=None) -> bytes:
         privkey, compressed = self.get_private_key(sequence, password)
         key = ecc.ECPrivkey(privkey)
-        return key.sign_message(message, compressed)
+        return bitcoin.ecdsa_sign_usermessage(key, message, is_compressed=compressed)
 
     def decrypt_message(self, sequence, message, password) -> bytes:
         privkey, compressed = self.get_private_key(sequence, password)
         ec = ecc.ECPrivkey(privkey)
-        decrypted = ec.decrypt_message(message)
+        decrypted = crypto.ecies_decrypt_message(ec, message)
         return decrypted
 
     def sign_transaction(self, tx, password):
@@ -231,9 +239,11 @@ class Software_KeyStore(KeyStore):
         # Raise if password is not correct.
         self.check_password(password)
         # Add private keys
-        keypairs = self._get_tx_derivations(tx)
-        for k, v in keypairs.items():
-            keypairs[k] = self.get_private_key(v, password)
+        keypairs = {}
+        pubkey_to_deriv_map = self._get_tx_derivations(tx)
+        for pubkey, deriv in pubkey_to_deriv_map.items():
+            privkey, is_compressed = self.get_private_key(deriv, password)
+            keypairs[pubkey] = privkey
         # Sign
         if keypairs:
             tx.sign(keypairs)
@@ -368,11 +378,11 @@ class Deterministic_KeyStore(Software_KeyStore):
     def format_seed(self, seed: str) -> str:
         pass
 
-    def add_seed(self, seed):
+    def add_seed(self, seed: str) -> None:
         if self.seed:
             raise Exception("a seed exists")
         self.seed = self.format_seed(seed)
-        self._seed_type = seed_type(seed) or None
+        self._seed_type = calc_seed_type(seed) or None
 
     def get_seed(self, password):
         if not self.has_seed():
@@ -571,7 +581,7 @@ class Xpub(MasterPublicKeyMixin):
             deriv_path=strpath,
         )
 
-    def add_key_origin_from_root_node(self, *, derivation_prefix: str, root_node: BIP32Node):
+    def add_key_origin_from_root_node(self, *, derivation_prefix: str, root_node: BIP32Node) -> None:
         assert self.xpub
         # try to derive ourselves from what we were given
         child_node1 = root_node.subkey_at_private_derivation(derivation_prefix)
@@ -629,6 +639,9 @@ class BIP32_KeyStore(Xpub, Deterministic_KeyStore):
         self.xpub = d.get('xpub')
         self.xprv = d.get('xprv')
 
+    def watching_only_keystore(self):
+        return BIP32_KeyStore({'xpub':self.xpub})
+
     def format_seed(self, seed):
         return ' '.join(seed.split())
 
@@ -671,18 +684,18 @@ class BIP32_KeyStore(Xpub, Deterministic_KeyStore):
     def is_watching_only(self):
         return self.xprv is None
 
-    def add_xpub(self, xpub):
+    def add_xpub(self, xpub: str) -> None:
         assert is_xpub(xpub)
         self.xpub = xpub
         root_fingerprint, derivation_prefix = bip32.root_fp_and_der_prefix_from_xkey(xpub)
         self.add_key_origin(derivation_prefix=derivation_prefix, root_fingerprint=root_fingerprint)
 
-    def add_xprv(self, xprv):
+    def add_xprv(self, xprv: str) -> None:
         assert is_xprv(xprv)
         self.xprv = xprv
         self.add_xpub(bip32.xpub_from_xprv(xprv))
 
-    def add_xprv_from_seed(self, bip32_seed, xtype, derivation):
+    def add_xprv_from_seed(self, bip32_seed: bytes, *, xtype: str, derivation: str) -> None:
         rootnode = BIP32Node.from_rootseed(bip32_seed, xtype=xtype)
         node = rootnode.subkey_at_private_derivation(derivation)
         self.add_xprv(node.to_xprv())
@@ -721,6 +734,9 @@ class Old_KeyStore(MasterPublicKeyMixin, Deterministic_KeyStore):
         self.mpk = d.get('mpk')
         self._root_fingerprint = None
 
+    def watching_only_keystore(self):
+        return Old_KeyStore({'mpk': self.mpk})
+
     def get_hex_seed(self, password):
         return pw_decode(self.seed, password, version=self.pw_hash_version).encode('utf8')
 
@@ -729,12 +745,12 @@ class Old_KeyStore(MasterPublicKeyMixin, Deterministic_KeyStore):
         d['mpk'] = self.mpk
         return d
 
-    def add_seed(self, seedphrase):
-        Deterministic_KeyStore.add_seed(self, seedphrase)
+    def add_seed(self, seed):
+        Deterministic_KeyStore.add_seed(self, seed)
         s = self.get_hex_seed(None)
         self.mpk = self.mpk_from_seed(s)
 
-    def add_master_public_key(self, mpk):
+    def add_master_public_key(self, mpk) -> None:
         self.mpk = mpk
 
     def format_seed(self, seed):
@@ -972,11 +988,13 @@ KeyStoreWithMPK = Union[KeyStore, MasterPublicKeyMixin]  # intersection really..
 AddressIndexGeneric = Union[Sequence[int], str]  # can be hex pubkey str
 
 
-def bip39_normalize_passphrase(passphrase):
+def bip39_normalize_passphrase(passphrase: str):
     return normalize('NFKD', passphrase or '')
 
-def bip39_to_seed(mnemonic, passphrase):
-    import hashlib, hmac
+
+def bip39_to_seed(mnemonic: str, *, passphrase: Optional[str]) -> bytes:
+    import hashlib
+    passphrase = passphrase or ""
     PBKDF2_ROUNDS = 2048
     mnemonic = normalize('NFKD', ' '.join(mnemonic.split()))
     passphrase = bip39_normalize_passphrase(passphrase)
@@ -1018,11 +1036,16 @@ def bip39_is_checksum_valid(
     return checksum == calculated_checksum, True
 
 
-def from_bip43_rootseed(root_seed, derivation, xtype=None):
+def from_bip43_rootseed(
+    root_seed: bytes,
+    *,
+    derivation: str,
+    xtype: Optional[str] = None,
+):
     k = BIP32_KeyStore({})
     if xtype is None:
         xtype = xtype_from_derivation(derivation)
-    k.add_xprv_from_seed(root_seed, xtype, derivation)
+    k.add_xprv_from_seed(root_seed, xtype=xtype, derivation=derivation)
     return k
 
 
@@ -1150,23 +1173,26 @@ def purpose48_derivation(account_id: int, xtype: str) -> str:
     return normalize_bip32_derivation(der)
 
 
-def from_seed(seed, passphrase, is_p2sh=False):
-    t = seed_type(seed)
+def from_seed(seed: str, *, passphrase: Optional[str], for_multisig: bool = False):
+    passphrase = passphrase or ""
+    t = calc_seed_type(seed)
     if t == 'old':
+        if passphrase:
+            raise Exception("'old'-type electrum seed cannot have passphrase")
         keystore = Old_KeyStore({})
         keystore.add_seed(seed)
     elif t in ['standard', 'segwit']:
         keystore = BIP32_KeyStore({})
         keystore.add_seed(seed)
         keystore.passphrase = passphrase
-        bip32_seed = Mnemonic.mnemonic_to_seed(seed, passphrase)
+        bip32_seed = Mnemonic.mnemonic_to_seed(seed, passphrase=passphrase)
         if t == 'standard':
             der = "m/"
             xtype = 'standard'
         else:
-            der = "m/1'/" if is_p2sh else "m/0'/"
-            xtype = 'p2wsh' if is_p2sh else 'p2wpkh'
-        keystore.add_xprv_from_seed(bip32_seed, xtype, der)
+            der = "m/1'/" if for_multisig else "m/0'/"
+            xtype = 'p2wsh' if for_multisig else 'p2wpkh'
+        keystore.add_xprv_from_seed(bip32_seed, xtype=xtype, derivation=der)
     else:
         raise BitcoinException('Unexpected seed type {}'.format(repr(t)))
     return keystore
