@@ -86,9 +86,9 @@ class SPV(NetworkJobOnDefaultServer):
             # if it's in the checkpoint region, we still might not have the header
             header = self.blockchain.read_header(tx_height)
             if header is None:
-                if tx_height < constants.net.max_checkpoint():
+                if tx_height <= constants.net.max_checkpoint():
                     # FIXME these requests are not counted (self._requests_sent += 1)
-                    await self.taskgroup.spawn(self.interface.request_chunk(tx_height, None, can_return_early=True))
+                    await self.taskgroup.spawn(self.interface.request_chunk(tx_height, can_return_early=True))
                 continue
             # request now
             self.logger.info(f'requested merkle {tx_hash}')
@@ -121,7 +121,7 @@ class SPV(NetworkJobOnDefaultServer):
         try:
             verify_tx_is_in_block(tx_hash, merkle_branch, pos, header, tx_height)
         except MerkleVerificationFailure as e:
-            if self.network.config.get("skipmerklecheck"):
+            if self.network.config.NETWORK_SKIPMERKLECHECK:
                 self.logger.info(f"skipping merkle proof check {tx_hash}")
             else:
                 self.logger.info(repr(e))
@@ -151,7 +151,7 @@ class SPV(NetworkJobOnDefaultServer):
         index = leaf_pos_in_tree
         for item in merkle_branch_bytes:
             if len(item) != 32:
-                raise MerkleVerificationFailure('all merkle branch items have to 32 bytes long')
+                raise MerkleVerificationFailure('all merkle branch items have to be 32 bytes long')
             inner_node = (item + h) if (index & 1) else (h + item)
             cls._raise_if_valid_tx(inner_node.hex())
             h = sha256d(inner_node)
