@@ -3,22 +3,26 @@ import os
 
 from typing import List, NamedTuple, Any, Dict, Optional, Tuple, TYPE_CHECKING
 
+from electrum.gui.messages import TERMS_OF_USE_LATEST_VERSION
+
 from electrum.i18n import _
 from electrum.interface import ServerAddr
 from electrum.keystore import hardware_keystore
 from electrum.logging import get_logger
+from electrum.network import ProxySettings
 from electrum.plugin import run_hook
 from electrum.slip39 import EncryptedSeed
 from electrum.storage import WalletStorage, StorageEncryptionVersion
 from electrum.wallet_db import WalletDB
 from electrum.bip32 import normalize_bip32_derivation, xpub_type
 from electrum import keystore, mnemonic, bitcoin
-from electrum.mnemonic import is_any_2fa_seed_type
+from electrum.mnemonic import is_any_2fa_seed_type, can_seed_have_passphrase
 
 if TYPE_CHECKING:
     from electrum.daemon import Daemon
     from electrum.plugin import Plugins
     from electrum.keystore import Hardware_KeyStore
+    from electrum.simple_config import SimpleConfig
 
 
 class WizardViewState(NamedTuple):
@@ -194,12 +198,188 @@ class AbstractWizard:
         return copy.deepcopy(self._current.wizard_data)
 
 
-class NewWalletWizard(AbstractWizard):
+class KeystoreWizard(AbstractWizard):
+
+    _logger = get_logger(__name__)
+
+    def __init__(self, plugins):
+        AbstractWizard.__init__(self)
+        self.plugins = plugins
+        self.navmap = {
+            'keystore_type': {
+                'next': self.on_keystore_type
+            },
+            'enter_seed': {
+                'next': 'enter_ext',
+                'accept': lambda d: None if self.wants_ext(d) else self.update_keystore(d),
+                'last': lambda d: not self.wants_ext(d),
+            },
+            'enter_ext': {
+                'accept': self.update_keystore,
+                'last': True
+            },
+            'choose_hardware_device': {
+                'next': self.on_hardware_device,
+            },
+        }
+
+    def maybe_master_pubkey(self, wizard_data):
+        self.update_keystore(wizard_data)
+
+    def check_multisig_constraints(self, wizard_data: dict) -> Tuple[bool, str]:
+        # called by GUI. overloaded in NewWalletWizard
+        return True, ''
+
+    def update_keystore(self, wizard_data):
+        wallet_type = wizard_data['wallet_type']
+        keystore = self.keystore_from_data(wallet_type, wizard_data)
+        self._result = keystore, (wizard_data['keystore_type'] == 'hardware')
+
+    def on_keystore_type(self, wizard_data: dict) -> str:
+        t = wizard_data['keystore_type']
+        return {
+            'haveseed': 'enter_seed',
+            'hardware': 'choose_hardware_device'
+        }.get(t)
+
+    def last_cosigner(self, wizard_data: dict) -> bool:
+        # one at a time
+        return True
+
+    def start(self, initial_data: dict = None) -> WizardViewState:
+        if initial_data is None:
+            initial_data = {}
+        self.reset()
+        start_view = 'keystore_type'
+        params = self.navmap[start_view].get('params', {})
+        self._current = WizardViewState(start_view, initial_data, params)
+        return self._current
+
+    # returns (sub)dict of current cosigner (or root if first)
+    def current_cosigner(self, wizard_data: dict) -> dict:
+        wdata = wizard_data
+        if wizard_data.get('wallet_type') == 'multisig' and 'multisig_current_cosigner' in wizard_data:
+            cosigner = wizard_data['multisig_current_cosigner']
+            wdata = wizard_data['multisig_cosigner_data'][str(cosigner)]
+        return wdata
+
+    def needs_derivation_path(self, wizard_data: dict) -> bool:
+        wdata = self.current_cosigner(wizard_data)
+        return 'seed_variant' in wdata and wdata['seed_variant'] in ['bip39', 'slip39']
+
+    def wants_ext(self, wizard_data: dict) -> bool:
+        wdata = self.current_cosigner(wizard_data)
+        return 'seed_variant' in wdata and wdata['seed_extend']
+
+    def is_multisig(self, wizard_data: dict) -> bool:
+        return wizard_data['wallet_type'] == 'multisig'
+
+    def is_hardware(self, wizard_data: dict) -> bool:
+        return wizard_data['keystore_type'] == 'hardware'
+
+    def wallet_password_view(self, wizard_data: dict) -> str:
+        if self.is_hardware(wizard_data) and wizard_data['wallet_type'] == 'standard':
+            return 'wallet_password_hardware'
+        return 'wallet_password'
+
+    def on_hardware_device(self, wizard_data: dict, new_wallet=True) -> str:
+        current_cosigner = self.current_cosigner(wizard_data)
+        _type, _info = current_cosigner['hardware_device']
+        run_hook('init_wallet_wizard', self)  # TODO: currently only used for hww, hook name might be confusing
+        plugin = self.plugins.get_plugin(_type)
+        return plugin.wizard_entry_for_device(_info, new_wallet=new_wallet)
+
+    def validate_seed(self, seed: str, seed_variant: str, wallet_type: str):
+        seed_type = ''
+        seed_valid = False
+        validation_message = ''
+        can_passphrase = True
+
+        if seed_variant == 'electrum':
+            seed_type = mnemonic.calc_seed_type(seed)
+            if seed_type != '':
+                seed_valid = True
+                can_passphrase = can_seed_have_passphrase(seed)
+        elif seed_variant == 'bip39':
+            is_checksum, is_wordlist = keystore.bip39_is_checksum_valid(seed)
+            validation_message = ('' if is_checksum else _('BIP39 checksum failed')) if is_wordlist else _('Unknown BIP39 wordlist')
+            if not bool(seed):
+                validation_message = ''
+            seed_type = 'bip39'
+            # bip39 always valid, even if checksum failed, see #8720
+            # however, reject empty string
+            seed_valid = bool(seed)
+        elif seed_variant == 'slip39':
+            # seed shares should be already validated by wizard page, we have a combined encrypted seed
+            if seed and isinstance(seed, EncryptedSeed):
+                seed_valid = True
+                seed_type = 'slip39'
+            else:
+                seed_valid = False
+        else:
+            raise Exception(f'unknown seed variant {seed_variant}')
+
+        # check if seed matches wallet type
+        if wallet_type == '2fa' and not is_any_2fa_seed_type(seed_type):
+            seed_valid = False
+        elif wallet_type == 'standard' and seed_type not in ['old', 'standard', 'segwit', 'bip39', 'slip39']:
+            seed_valid = False
+        elif wallet_type == 'multisig' and seed_type not in ['standard', 'segwit', 'bip39', 'slip39']:
+            seed_valid = False
+
+        self._logger.debug(f'seed verified: {seed_valid}, type={seed_type!r}, validation_message={validation_message}')
+
+        return seed_valid, seed_type, validation_message, can_passphrase
+
+    def keystore_from_data(self, wallet_type: str, data: dict):
+        if data['keystore_type'] in ['createseed', 'haveseed'] and 'seed' in data:
+            seed_extension = data['seed_extra_words'] if data['seed_extend'] else ''
+            if data['seed_variant'] == 'electrum':
+                for_multisig = wallet_type in ['multisig']
+                return keystore.from_seed(data['seed'], passphrase=seed_extension, for_multisig=for_multisig)
+            elif data['seed_variant'] == 'bip39':
+                root_seed = keystore.bip39_to_seed(data['seed'], passphrase=seed_extension)
+                derivation = normalize_bip32_derivation(data['derivation_path'])
+                if wallet_type == 'multisig':
+                    script = data['script_type'] if data['script_type'] != 'p2sh' else 'standard'
+                else:
+                    script = data['script_type'] if data['script_type'] != 'p2pkh' else 'standard'
+                return keystore.from_bip43_rootseed(root_seed, derivation=derivation, xtype=script)
+            elif data['seed_variant'] == 'slip39':
+                root_seed = data['seed'].decrypt(seed_extension)
+                derivation = normalize_bip32_derivation(data['derivation_path'])
+                if wallet_type == 'multisig':
+                    script = data['script_type'] if data['script_type'] != 'p2sh' else 'standard'
+                else:
+                    script = data['script_type'] if data['script_type'] != 'p2pkh' else 'standard'
+                return keystore.from_bip43_rootseed(root_seed, derivation=derivation, xtype=script)
+            else:
+                raise Exception('Unsupported seed variant %s' % data['seed_variant'])
+        elif data['keystore_type'] == 'masterkey' and 'master_key' in data:
+            return keystore.from_master_key(data['master_key'])
+        elif data['keystore_type'] == 'hardware':
+            return self.hw_keystore(data)
+        else:
+            raise Exception('no seed or master_key in data')
+
+    def hw_keystore(self, data: dict) -> 'Hardware_KeyStore':
+        return hardware_keystore({
+            'type': 'hardware',
+            'hw_type': data['hw_type'],
+            'derivation': data['derivation_path'],
+            'root_fingerprint': data['root_fingerprint'],
+            'xpub': data['master_key'],
+            'label': data['label'],
+            'soft_device_id': data['soft_device_id']
+        })
+
+
+class NewWalletWizard(KeystoreWizard):
 
     _logger = get_logger(__name__)
 
     def __init__(self, daemon: 'Daemon', plugins: 'Plugins'):
-        AbstractWizard.__init__(self)
+        KeystoreWizard.__init__(self, plugins)
         self.navmap = {
             'wallet_name': {
                 'next': 'wallet_type'
@@ -271,6 +451,8 @@ class NewWalletWizard(AbstractWizard):
         }
         self._daemon = daemon
         self.plugins = plugins
+        # todo: load only if needed, like hw plugins
+        self.plugins.load_plugin_by_name('trustedcoin')
 
     def start(self, initial_data: dict = None) -> WizardViewState:
         if initial_data is None:
@@ -284,25 +466,6 @@ class NewWalletWizard(AbstractWizard):
     def is_single_password(self) -> bool:
         raise NotImplementedError()
 
-    # returns (sub)dict of current cosigner (or root if first)
-    def current_cosigner(self, wizard_data: dict) -> dict:
-        wdata = wizard_data
-        if wizard_data.get('wallet_type') == 'multisig' and 'multisig_current_cosigner' in wizard_data:
-            cosigner = wizard_data['multisig_current_cosigner']
-            wdata = wizard_data['multisig_cosigner_data'][str(cosigner)]
-        return wdata
-
-    def needs_derivation_path(self, wizard_data: dict) -> bool:
-        wdata = self.current_cosigner(wizard_data)
-        return 'seed_variant' in wdata and wdata['seed_variant'] in ['bip39', 'slip39']
-
-    def wants_ext(self, wizard_data: dict) -> bool:
-        wdata = self.current_cosigner(wizard_data)
-        return 'seed_variant' in wdata and wdata['seed_extend']
-
-    def is_multisig(self, wizard_data: dict) -> bool:
-        return wizard_data['wallet_type'] == 'multisig'
-
     def on_wallet_type(self, wizard_data: dict) -> str:
         t = wizard_data['wallet_type']
         return {
@@ -321,21 +484,6 @@ class NewWalletWizard(AbstractWizard):
             'hardware': 'choose_hardware_device'
         }.get(t)
 
-    def is_hardware(self, wizard_data: dict) -> bool:
-        return wizard_data['keystore_type'] == 'hardware'
-
-    def wallet_password_view(self, wizard_data: dict) -> str:
-        if self.is_hardware(wizard_data) and wizard_data['wallet_type'] == 'standard':
-            return 'wallet_password_hardware'
-        return 'wallet_password'
-
-    def on_hardware_device(self, wizard_data: dict, new_wallet=True) -> str:
-        current_cosigner = self.current_cosigner(wizard_data)
-        _type, _info = current_cosigner['hardware_device']
-        run_hook('init_wallet_wizard', self)  # TODO: currently only used for hww, hook name might be confusing
-        plugin = self.plugins.get_plugin(_type)
-        return plugin.wizard_entry_for_device(_info, new_wallet=new_wallet)
-
     def on_have_or_confirm_seed(self, wizard_data: dict) -> str:
         if self.needs_derivation_path(wizard_data):
             return 'script_and_derivation'
@@ -412,35 +560,6 @@ class NewWalletWizard(AbstractWizard):
                 return True
         return False
 
-    def keystore_from_data(self, wallet_type: str, data: dict):
-        if data['keystore_type'] in ['createseed', 'haveseed'] and 'seed' in data:
-            if data['seed_variant'] == 'electrum':
-                return keystore.from_seed(data['seed'], passphrase=data['seed_extra_words'], for_multisig=True)
-            elif data['seed_variant'] == 'bip39':
-                root_seed = keystore.bip39_to_seed(data['seed'], passphrase=data['seed_extra_words'])
-                derivation = normalize_bip32_derivation(data['derivation_path'])
-                if wallet_type == 'multisig':
-                    script = data['script_type'] if data['script_type'] != 'p2sh' else 'standard'
-                else:
-                    script = data['script_type'] if data['script_type'] != 'p2pkh' else 'standard'
-                return keystore.from_bip43_rootseed(root_seed, derivation=derivation, xtype=script)
-            elif data['seed_variant'] == 'slip39':
-                root_seed = data['seed'].decrypt(data['seed_extra_words'])
-                derivation = normalize_bip32_derivation(data['derivation_path'])
-                if wallet_type == 'multisig':
-                    script = data['script_type'] if data['script_type'] != 'p2sh' else 'standard'
-                else:
-                    script = data['script_type'] if data['script_type'] != 'p2pkh' else 'standard'
-                return keystore.from_bip43_rootseed(root_seed, derivation=derivation, xtype=script)
-            else:
-                raise Exception('Unsupported seed variant %s' % data['seed_variant'])
-        elif data['keystore_type'] == 'masterkey' and 'master_key' in data:
-            return keystore.from_master_key(data['master_key'])
-        elif data['keystore_type'] == 'hardware':
-            return self.hw_keystore(data)
-        else:
-            raise Exception('no seed or master_key in data')
-
     def is_current_cosigner_hardware(self, wizard_data: dict) -> bool:
         cosigner_data = self.current_cosigner(wizard_data)
         cosigner_is_hardware = cosigner_data == wizard_data and wizard_data['keystore_type'] == 'hardware'
@@ -482,48 +601,39 @@ class NewWalletWizard(AbstractWizard):
 
         return multisig_keys_valid, user_info
 
-    def validate_seed(self, seed: str, seed_variant: str, wallet_type: str):
-        seed_type = ''
-        seed_valid = False
+    def validate_master_key(self, key: str, wallet_type: str):
+        # TODO: deduplicate with master key check in create_storage()
         validation_message = ''
-        can_passphrase = True
+        key_valid = False
 
-        if seed_variant == 'electrum':
-            seed_type = mnemonic.seed_type(seed)
-            if seed_type != '':
-                seed_valid = True
-            if seed_type in ['old', '2fa']:
-                can_passphrase = False
-        elif seed_variant == 'bip39':
-            is_checksum, is_wordlist = keystore.bip39_is_checksum_valid(seed)
-            validation_message = ('' if is_checksum else _('BIP39 checksum failed')) if is_wordlist else _('Unknown BIP39 wordlist')
-            if not bool(seed):
-                validation_message = ''
-            seed_type = 'bip39'
-            # bip39 always valid, even if checksum failed, see #8720
-            # however, reject empty string
-            seed_valid = bool(seed)
-        elif seed_variant == 'slip39':
-            # seed shares should be already validated by wizard page, we have a combined encrypted seed
-            if seed and isinstance(seed, EncryptedSeed):
-                seed_valid = True
-                seed_type = 'slip39'
-            else:
-                seed_valid = False
+        if not keystore.is_master_key(key):
+            validation_message = _('Not a master key')
         else:
-            raise Exception(f'unknown seed variant {seed_variant}')
+            k = keystore.from_master_key(key)
+            if wallet_type == 'standard':
+                if isinstance(k, keystore.Xpub):  # has bip32 xpub
+                    t1 = xpub_type(k.xpub)
+                    if t1 not in ['standard', 'p2wpkh', 'p2wpkh-p2sh']:  # disallow Ypub/Zpub
+                        validation_message = '%s: %s' % (_('Wrong key type'), t1)
+                    else:
+                        key_valid = True
+                elif isinstance(k, keystore.Old_KeyStore):
+                    key_valid = True
+                else:
+                    self._logger.error(f"unexpected keystore type: {type(keystore)}")
+            elif wallet_type == 'multisig':
+                if not isinstance(k, keystore.Xpub):  # old mpk?
+                    validation_message = '%s: %s' % (_('Wrong key type'), "not bip32")
+                t1 = xpub_type(k.xpub)
+                if t1 not in ['standard', 'p2wsh', 'p2wsh-p2sh']:  # disallow ypub/zpub
+                    validation_message = '%s: %s' % (_('Wrong key type'), t1)
+                else:
+                    key_valid = True
+            else:
+                validation_message = '%s: %s' % (_('Unsupported wallet type'), wallet_type)
+                self._logger.error(f'Unsupported wallet type: {wallet_type}')
 
-        # check if seed matches wallet type
-        if wallet_type == '2fa' and not is_any_2fa_seed_type(seed_type):
-            seed_valid = False
-        elif wallet_type == 'standard' and seed_type not in ['old', 'standard', 'segwit', 'bip39', 'slip39']:
-            seed_valid = False
-        elif wallet_type == 'multisig' and seed_type not in ['standard', 'segwit', 'bip39', 'slip39']:
-            seed_valid = False
-
-        self._logger.debug(f'seed verified: {seed_valid}, type={seed_type!r}, validation_message={validation_message}')
-
-        return seed_valid, seed_type, validation_message, can_passphrase
+        return key_valid, validation_message
 
     def create_storage(self, path: str, data: dict):
         assert data['wallet_type'] in ['standard', '2fa', 'imported', 'multisig']
@@ -549,15 +659,16 @@ class NewWalletWizard(AbstractWizard):
                 for addr in data['address_list'].split():
                     addresses[addr] = {}
         elif data['keystore_type'] in ['createseed', 'haveseed']:
+            seed_extension = data['seed_extra_words'] if data['seed_extend'] else ''
             if data['seed_type'] in ['old', 'standard', 'segwit']:
                 self._logger.debug('creating keystore from electrum seed')
-                k = keystore.from_seed(data['seed'], passphrase=data['seed_extra_words'], for_multisig=data['wallet_type'] == 'multisig')
+                k = keystore.from_seed(data['seed'], passphrase=seed_extension, for_multisig=data['wallet_type'] == 'multisig')
             elif data['seed_type'] in ['bip39', 'slip39']:
                 self._logger.debug('creating keystore from %s seed' % data['seed_type'])
                 if data['seed_type'] == 'bip39':
-                    root_seed = keystore.bip39_to_seed(data['seed'], passphrase=data['seed_extra_words'])
+                    root_seed = keystore.bip39_to_seed(data['seed'], passphrase=seed_extension)
                 else:
-                    root_seed = data['seed'].decrypt(data['seed_extra_words'])
+                    root_seed = data['seed'].decrypt(seed_extension)
                 derivation = normalize_bip32_derivation(data['derivation_path'])
                 if data['wallet_type'] == 'multisig':
                     script = data['script_type'] if data['script_type'] != 'p2sh' else 'standard'
@@ -598,16 +709,20 @@ class NewWalletWizard(AbstractWizard):
         else:
             raise Exception('unsupported/unknown keystore_type %s' % data['keystore_type'])
 
-        if data['encrypt']:
+        if data['password']:
             if k and k.may_have_password():
                 k.update_password(None, data['password'])
-            enc_version = StorageEncryptionVersion.USER_PASSWORD
-            if data.get('keystore_type') == 'hardware' and data['wallet_type'] == 'standard':
+
+        if data['encrypt']:
+            if data.get('xpub_encrypt'):
+                assert data.get('keystore_type') == 'hardware' and data['wallet_type'] == 'standard'
                 enc_version = StorageEncryptionVersion.XPUB_PASSWORD
+            else:
+                enc_version = StorageEncryptionVersion.USER_PASSWORD
             storage.set_password(data['password'], enc_version=enc_version)
 
         db = WalletDB('', storage=storage, upgrade=True)
-        db.set_keystore_encryption(bool(data['password']) and data['encrypt'])
+        db.set_keystore_encryption(bool(data['password']))
 
         db.put('wallet_type', data['wallet_type'])
 
@@ -646,22 +761,11 @@ class NewWalletWizard(AbstractWizard):
             db.put('addresses', addresses)
 
         if k and k.can_have_deterministic_lightning_xprv():
-            db.put('lightning_xprv', k.get_lightning_xprv(data['password'] if data['encrypt'] else None))
+            db.put('lightning_xprv', k.get_lightning_xprv(data['password']))
 
         db.load_plugins()
         db.write()
 
-    def hw_keystore(self, data: dict) -> 'Hardware_KeyStore':
-        return hardware_keystore({
-            'type': 'hardware',
-            'hw_type': data['hw_type'],
-            'derivation': data['derivation_path'],
-            'root_fingerprint': data['root_fingerprint'],
-            'xpub': data['master_key'],
-            'label': data['label'],
-            'soft_device_id': data['soft_device_id']
-        })
-
 
 class ServerConnectWizard(AbstractWizard):
 
@@ -694,15 +798,15 @@ class ServerConnectWizard(AbstractWizard):
             return
         self._logger.debug(f'configuring proxy: {proxy_settings!r}')
         net_params = self._daemon.network.get_parameters()
-        if not proxy_settings['enabled']:
-            proxy_settings = None
-        net_params = net_params._replace(proxy=proxy_settings, auto_connect=bool(wizard_data['autoconnect']))
+        proxy = ProxySettings.from_dict(proxy_settings)
+        net_params = net_params._replace(proxy=proxy, auto_connect=bool(wizard_data['autoconnect']))
         self._daemon.network.run_from_another_thread(self._daemon.network.set_parameters(net_params))
 
     def do_configure_server(self, wizard_data: dict):
         self._logger.debug(f'configuring server: {wizard_data!r}')
         net_params = self._daemon.network.get_parameters()
         server = ''
+        oneserver = wizard_data.get('one_server', False)
         if not wizard_data['autoconnect']:
             try:
                 server = ServerAddr.from_str_with_inference(wizard_data['server'])
@@ -710,7 +814,7 @@ class ServerConnectWizard(AbstractWizard):
                     raise Exception('failed to parse server %s' % wizard_data['server'])
             except Exception:
                 return
-        net_params = net_params._replace(server=server, auto_connect=wizard_data['autoconnect'])
+        net_params = net_params._replace(server=server, auto_connect=wizard_data['autoconnect'], oneserver=oneserver)
         self._daemon.network.run_from_another_thread(self._daemon.network.set_parameters(net_params))
 
     def do_configure_autoconnect(self, wizard_data: dict):
@@ -727,3 +831,30 @@ class ServerConnectWizard(AbstractWizard):
         params = self.navmap[start_view].get('params', {})
         self._current = WizardViewState(start_view, initial_data, params)
         return self._current
+
+
+class TermsOfUseWizard(AbstractWizard):
+
+    _logger = get_logger(__name__)
+
+    def __init__(self, config: 'SimpleConfig'):
+        AbstractWizard.__init__(self)
+        self._config = config
+        self.navmap = {
+            'terms_of_use': {
+                'accept': self.accept_terms_of_use,
+                'last': True,
+            },
+        }
+
+    def accept_terms_of_use(self, _):
+        self._config.TERMS_OF_USE_ACCEPTED = TERMS_OF_USE_LATEST_VERSION
+
+    def start(self, initial_data: dict = None) -> WizardViewState:
+        if initial_data is None:
+            initial_data = {}
+        self.reset()
+        start_view = 'terms_of_use'
+        params = self.navmap[start_view].get('params', {})
+        self._current = WizardViewState(start_view, initial_data, params)
+        return self._current
